      program make_grid
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to generate the main input data file for program "planet_EMC" (file "grid.in")
c     See the end of this file (section "production of the grid file") for details on the
c     format used.
c
c     Values have to be given for the following variables:
c       + Ntheta: number of columns on latitude
c       + Nphi: number of columns on longitude
c       + Nb_gas: number of narrowband spectral intervals for gas data
c       + Nq: quadrature order
c       + Nzones: number of special emissivity zones
c       + Nz(itheta,iphi): number of cells for each (itheta,iphi) column
c       + z(iz,itheta,iphi): altitudes (in meters) for each interface 'iz'
c         of each (itheta,iphi) column.
c         Note: z(0,itheta,iphi) is the altitude of the ground, at the center
c         of each (itheta,iphi) column. This value must be lower than values of the
c         altitude at the top of the first gas cell for adjacent columns z(1,itheta+/-1,iphi+/-1)
c       + theta(i): the Ntheta+1 values of latitude (in radians) that define the latitudinal extension
c         of each column. theta(0) has to be -pi/2, and theta(Ntheta) has to be pi/2.
c       + phi(i): the Nphi+1 values of longitude (in radians) that define the longitudinal extension
c         of each column. phi(0) has to be 0.0 and phi(Nphi) has to be 2pi.
c       + z_mc(i,itheta,iphi): the altitude at mass center (in meters) of each gas layer i, for each
c         (itheta,iphi) column.
c       + Tground(itheta,iphi): the temperature (in Kelvins) of ground for each (itheta,iphi) column.
c       + T(i,itheta,iphi): the temperature (in Kelvins) profile within the gas, for each (itheta,iphi) column:
c         - T(0,itheta,iphi) is the temperature of the gas at ground level
c         - T(i,itheta,iphi) for i=1 to Nz(itheta,iphi) is the temperature of the gas
c           at mass center of cell index i
c         - T(Nz(itheta,iphi)+1,itheta,iphi) is the temperature of the gas at space level
c       + Tspace(itheta,iphi): the temperature (in Kelvins) of space for each (itheta,iphi) column.
c       + lambda_gas_lo(band): the lower value of wavelength (in micrometers) for each one of the Nb
c         spectral narrowband intervals defined for gas data.
c       + lambda_gas_hi(band): the higher value of wavelength (in micrometers) for each one of the Nb
c         spectral narrowband intervals defined for gas data.
c       + w_quad(quad): the Nq quadrature weights
c       + ground_emissivity(itheta,iphi,band): emissivity of the ground, for each (itheta,iphi) column,
c         and for each gas narrowband spectral interval.
c       + space_emissivity(itheta,iphi,band): emissivity of space, for each (itheta,iphi) column,
c         and for each gas narrowband spectral interval.
c       + ka_gas(ir,itheta,iphi,band,quad): gas absorption coefficient (in inverse meters)
c         that has to be provided for every gas cell, for every column, and for every quadrature element.
c       + ks_gas(ir,itheta,iphi,band,quad): gas scattering coefficient (in inverse meters)
c         that has to be provided for every gas cell, for every column, and for every quadrature element.
c       + emissivity_zone(izone,j): there are 4 values that must be provided for every zone 'izone':
c         - emissivity_zone(izone,1): minimum latitude (radians) of the zone
c         - emissivity_zone(izone,2): maximum latitude (radians) of the zone
c         - emissivity_zone(izone,3): minimum longitude (radians) of the zone
c         - emissivity_zone(izone,4): maximum longitude (radians) of the zone
c       + emissivity(izone,band): the value of emissivity within each special zone, for each
c         gas narrowband spectral interval.
c       + n_ref(ir,itheta,iphi,band): the value of the refraction index, that has to be provided
c         for every gas cell, for every column, and for every gas narrowband spectral interval.
c

c     Variables definition
      integer Nb_gas,Nq,Ntheta,Nphi,Nz(1:Ntheta_mx,1:Nphi_mx)
      integer band,quad,ir,itheta,iphi,j
      integer Ns_theta,Ns_phi
      integer Nzones,izone
      double precision z(0:Nr_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
      double precision ka_gas(1:Nr_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision ks_gas(1:Nr_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision Tground(1:Ntheta_mx,1:Nphi_mx)
      double precision T(0:Nr_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision z_mc(1:Nr_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision Tspace(1:Ntheta_mx,1:Nphi_mx)
      double precision n_ref(1:Nr_mx,1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision emissivity_zone(1:Nzones_mx,4)
      double precision emissivity(1:Nzones_mx,1:Nb_gas_mx)
      integer ok,i
      double precision H
      integer strlen
      character*100 label,grid_file
      label='make_grid'
      grid_file='./grid.in'

c     -------------------------------------------------------------------------------------------------------------
c     3D spherical grid
c     -------------------------------------------------------------------------------------------------------------

c     The purpose of this section is to retrieve the altitudes that will define each gas cell
c     (altitudes of their interfaces), for each column, and to define the longitude/latitude grid
c
c     Latitude discretization - use a constant angular step, and 4 angular sectors per [0;pi/2] latitudinal interval
      call latitude_grid(1,4,Ntheta,theta)
c     Longitude discretization - use 2 angular sectors per [0;pi/2] longitudinal interval
      call longitude_grid(2,Nphi,phi)
c     Vertical grid: use 10 gas layers for every column
c     variable H is used to define the total height of the atmosphere
      H=100.0D+3 ! m
      do itheta=1,Ntheta
         do iphi=1,Nphi
            Nz(itheta,iphi)=10
            do i=0,Nz(itheta,iphi)
               z(i,itheta,iphi)=H*dble(i)/dble(Nz(itheta,iphi)) ! meters
            enddo ! i
         enddo ! iphi
      enddo ! itheta

c     -------------------------------------------------------------------------------------------------------------
c     Temperature profiles - These are currently not used by the code !
c     -------------------------------------------------------------------------------------------------------------
      
c     set temperature profile to zero for each column
      do itheta=1,Ntheta
         do iphi=1,Nphi
            Tground(itheta,iphi)=0.0D+0
            T(0,itheta,iphi)=0.0D+0
            do i=1,Nz(itheta,iphi)
               T(i,itheta,iphi)=0.0D+0
            enddo
            T(Nz(itheta,iphi)+1,itheta,iphi)=0.0D+0
            Tspace(itheta,iphi)=0.0D+0
         enddo
      enddo

c     coherence check: all space altitudes must be the same
      call check_space_altitudes(Nz,Ntheta,Nphi,z)
      write(*,*) 'coherence check for space altitudes - ok'
c     coherence check: the altitude at ground level for each column must be lower than the altitude
c     at the top of the first gas layer, for any adjacent column
      call check_zground_consistency(Nz,Ntheta,Nphi,z,ok)
      if (ok.eq.0) then
         write(*,*) 'Error from program ',
     &        label(1:strlen(label)),':'
         write(*,*) 'ground altitudes do not match dependencies'
         stop
      else
         write(*,*) 'ground altitude consistency check - ok'
      endif

c     -------------------------------------------------------------------------------------------------------------
c     Spectral data
c     -------------------------------------------------------------------------------------------------------------

c     "Nb_gas" is the number of narrowband spectral intervals, for gas data
      Nb_gas=20
c     "Nq" is the quadrature order
      Nq=8
c     "lambda_gas_lo" holds the values of the lower limit (in m)
c     of each narrowband spectral interval defined for gas data
      lambda_gas_lo(1)=1.5D+0 ! m
      do band=2,Nb_gas
         lambda_gas_lo(band)=lambda_gas_lo(band-1)+0.5D+0 ! m
         lambda_gas_hi(band-1)=lambda_gas_lo(band) ! m
      enddo
c     "lambda_gas_lo" holds the values of the higher limit (in m)
c     of each narrowband spectral interval defined for gas data
      lambda_gas_hi(Nb_gas)=lambda_gas_lo(Nb_gas)+0.5D+0 ! m
c     "w_quad" holds the quadrature weights
      do quad=1,Nq
         w_quad(quad)=1.0D+0/dble(Nq)
      enddo
c     "ka_gas" is the absorption coefficient (in inverse meters) for each gas cell,
c     for each gas narrowband, for each quadrature element.
c     "ks_gas" is the scattering coefficient (in inverse meters) for each gas cell.
c     for each gas narrowband, for each quadrature element.
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nz(itheta,iphi)
               do band=1,Nb_gas
                  do quad=1,Nq
                     ka_gas(ir,itheta,iphi,band,quad)=1.0D-5 ! m
                     ks_gas(ir,itheta,iphi,band,quad)=2.0D-5 ! m
                  enddo         ! quad
               enddo            ! band
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta

c     -------------------------------------------------------------------------------------------------------------
c     Emissivities
c     -------------------------------------------------------------------------------------------------------------

c     "ground_emissivity" is the emissivity of the ground, for each column, in each gas narrowband
c     "spaec_emissivity" is the emissivity of space (probably 1, but not necessarily), for each column,
c     in each gas narrowband
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do band=1,Nb_gas
               ground_emissivity(itheta,iphi,band)=0.0D+0
               space_emissivity(itheta,iphi,band)=1.0D+0
            enddo ! band
         enddo ! iphi
      enddo ! itheta

c     SPECIAL ZONES

c     "Nzones" is the number of special zones
      Nzones=1
c     spatial definition of each special zone
      emissivity_zone(1,1)=0.1D+0 ! minimum latitude for special zone number 1 (rad)
      emissivity_zone(1,2)=0.3D+0 ! maximum latitude for special zone number 1 (rad)
      emissivity_zone(1,3)=0.5D+0 ! minimum longitude for special zone number 1 (rad)
      emissivity_zone(1,4)=1.3D+0 ! maximum longitude for special zone number 1 (rad)
c     values of emissivity for each special zone
      do band=1,Nb_gas
         emissivity(1,band)=0.65D+0 ! value of emissivity for zone number 1
      enddo
      
c     refraction indexes
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nz(itheta,iphi)
               do band=1,Nb_gas
                  n_ref(ir,itheta,iphi,band)=1.0D+0
               enddo            ! band
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta


c     -------------------------------------------------------------------------------------------------------------
c     Production of the grid file
c     -------------------------------------------------------------------------------------------------------------

      write(*,*) 'now writing data file'

      open(10,file=grid_file(1:strlen(grid_file)))
c     Number of cells along the vertical, along the latitude, along the longitude,
c     number of gas narrowband spectral intervals, quadrature order, 
c     number of special emissivity zones
      write(10,*) Ntheta,Nphi,Nb_gas,Nq,Nzones
      do itheta=1,Ntheta
         do iphi=1,Nphi
            write(10,*) Nz(itheta,iphi)
c     limits in terms of radiuses
            write(10,50) (z(ir,itheta,iphi),ir=0,Nz(itheta,iphi)) ! m
         enddo ! iphi
      enddo ! itheta
c     limits in latitude
      write(10,50) (theta(itheta),itheta=0,Ntheta) ! rad
c     limits in longitude
      write(10,50) (phi(iphi),iphi=0,Nphi) ! rad
c     altitudes of mass center
      do itheta=1,Ntheta
         do iphi=1,Nphi
            write(10,50) (z_mc(ir,itheta,iphi),ir=1,Nz(itheta,iphi)) ! m
         enddo ! iphi
      enddo ! itheta
c     temperature profiles
      do itheta=1,Ntheta
         do iphi=1,Nphi
            write(10,50) Tground(itheta,iphi), ! K
     &           (T(ir,itheta,iphi),ir=0,Nz(itheta,iphi)+1), ! K
     &           Tspace(itheta,iphi) ! K
         enddo ! iphi
      enddo ! itheta
c     spectral stuff:
c     lower limits of gas data narrowband intervals
      write(10,50) (lambda_gas_lo(band),band=1,Nb_gas) ! m
c     higher limits of gas data narrowband intervals
      write(10,50) (lambda_gas_hi(band),band=1,Nb_gas) ! m
c     quadrature weights
      write(10,50) (w_quad(quad),quad=1,Nq)
c     emissivities...
      do itheta=1,Ntheta
         do iphi=1,Nphi
c     ... of ground
            write(10,50) (ground_emissivity(itheta,iphi,band),
     &           band=1,Nb_gas)
c     ... and of space
            write(10,50) (space_emissivity(itheta,iphi,band),
     &           band=1,Nb_gas)
         enddo ! iphi
      enddo ! itheta
c     absorption and scattering coefficients, for each gas cell, and
c     for each narrowband interval / quadrature element
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nz(itheta,iphi)
               do band=1,Nb_gas
                  write(10,50) (ka_gas(ir,itheta,iphi,band,quad),
     &                 quad=1,Nq) ! m^{-1}
                  write(10,50) (ks_gas(ir,itheta,iphi,band,quad),
     &                 quad=1,Nq) ! m^{-1}
               enddo            ! band
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta
c     special emissivity zones data:
c     spatial extension
      do izone=1,Nzones
         write(10,50) (emissivity_zone(izone,j),j=1,4) ! rad
      enddo
c     and values of emissivity, for each gas narrowband spectral interval
      do izone=1,Nzones
         write(10,50) (emissivity(izone,band),band=1,Nb_gas)
      enddo
c     refraction indexes
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do band=1,Nb_gas
               write(10,50) (n_ref(ir,itheta,iphi,band),
     &              ir=1,Nz(itheta,iphi))
            enddo ! band
         enddo ! iphi
      enddo ! itheta
 666  continue

      close(10)
      
      write(*,*) 'Output data file successfuly generated:'
      write(*,*) grid_file(1:strlen(grid_file))

      end
