c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      double precision function F_wavnb(nu,T)
      implicit none
c
c     Purpose: to compute the fraction of energy between 0 and wavenumber "nu"
c

c     Inputs:
c       + nu: wavenumber [cm^{-1}]
c       + T: temperature [K]
c
c     Output:
c       + F: fraction of energy between 0 and nu
c
      double precision nu,T
      double precision sigmaT4
      double precision nu_min
      parameter(nu_min=1.0D-5)
      double precision planck_int_wavnb
      integer strlen
      character*100 label
      label='subroutine F_wavnb'

      if (nu.le.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'nu=',nu
         stop
      endif

      if (T.lt.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      F_wavnb=planck_int_wavnb(nu_min,nu,T)/sigmaT4(T)

      if (F_wavnb.gt.1.0D+0) then
         F_wavnb=1.0D+0
      endif

      return
      end



      double precision function F_wavlng(lambda,T)
      implicit none
c
c     Purpose: to compute the fraction of energy between 0 and wavenlength "lambda"
c

c     Inputs:
c       + nu: wavenlength [m]
c       + T: temperature [K]
c
c     Output:
c       + F: fraction of energy between 0 and lambda
c
      double precision lambda,T
      double precision sigmaT4
      double precision lambda_min
      parameter(lambda_min=1.0D-5)
      double precision planck_int_wavlng
      integer strlen
      character*100 label
      label='subroutine F_wavlng'

      if (lambda.le.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'lambda=',lambda
         stop
      endif

      if (T.lt.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      F_wavlng=planck_int_wavlng(lambda_min,lambda,T)/sigmaT4(T)

      if (F_wavlng.gt.1.0D+0) then
         F_wavlng=1.0D+0
      endif

      return
      end




      double precision function sigmaT4(T)
      implicit none
c
c     Purpose: to compute the total energy emitted by a blackbody at temperature T
c
c     Input:
c       + T: temperature [K]
c
c     Output:
c       + sigmaT4 [W/m/sr]
c
      include 'param.inc'
      double precision T
      integer strlen
      character*100 label
      label='subroutine sigmaT4'

      if (T.lt.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      sigmaT4=Pl*T**4.0D+0/pi ! W/m/sr

      return
      end




      double precision function planck_int_wavnb(nu_lo,nu_hi,T)
      implicit none
c
c     Purpose: to compute the Planck function, integrated between two
c     values of the wavenumber
c
c     Inputs:
c       + nu_lo: lower value of wavenumber [cm^{-1}]
c       + nu_hi: higer value of wavenumber [cm^{-1}]
c       + T: temperature [K]
c
c     Output:
c       + planck_int_wavnb: Planck function at temperature T, integrated between
c            nu_lo and nu_hi [W//m/sr]
c

c     inputs
      double precision nu_lo,nu_hi,T
c     temp
      integer strlen
      real PLKAVG
      character*100 label
      label='subroutine planck_int_wavnb'

      if (nu_hi.lt.nu_lo) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameters:'
         write(*,*) 'nu_lo=',nu_lo,' > nu_hi=',nu_hi
         stop
      endif

      if (T.lt.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      planck_int_wavnb=PLKAVG(real(nu_lo),real(nu_hi),real(T)) ! W/m/sr

      return
      end




      double precision function planck_int_wavlng(lambda_lo,lambda_hi,
     &     T)
      implicit none
c
c     Purpose: to compute the Planck function, integrated between two
c     values of the wavelength
c
c     Inputs:
c       + lambda_lo: lower value of wavelength [m]
c       + lambda_hi: higer value of wavelength [m]
c       + T: temperature [K]
c
c     Output:
c       + planck_int_wavlng: Planck function at temperature T, integrated between
c            lambda_lo and lambda_hi [W//m/sr]
c

c     inputs
      double precision lambda_lo,lambda_hi,T
c     temp
      integer strlen
      double precision planck_int_wavnb
      double precision nu_lo,nu_hi
      character*100 label
      label='subroutine planck_int_wavlng'

      if (lambda_hi.lt.lambda_lo) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameters:'
         write(*,*) 'lambda_lo=',lambda_lo,' > lambda_hi=',lambda_hi
         stop
      endif

      if (T.lt.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      call lambda2nu(lambda_lo,nu_hi)
      call lambda2nu(lambda_hi,nu_lo)
      planck_int_wavlng=planck_int_wavnb(nu_lo,nu_hi,T) ! W/m/sr

      return
      end
