c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c

      subroutine sort_direction(dim,Ntheta,Nphi,theta,phi,u,
     &     itheta,iphi)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify which angular sector direction "u" belongs to
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: number of cells on the column grid
c       + theta,phi: angular limits of columns
c       + u: direction (cartesian coordinates)
c
c     Outputs:
c       + (itheta,iphi): indexes of the angular sector direction "u" belongs to
c     

c     Input
      integer dim
      integer Ntheta,Nphi
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision u(1:Nmat_mx)
c     Output
      integer itheta,iphi
c     temp
      double precision us(1:Nmat_mx)
      integer cf,i
      integer strlen
      character*100 label
      label='subroutine sort_direction'

      call cart2spher(u(1),u(2),u(3),us(1),us(2),us(3))

      call identify_column(us,Ntheta,Nphi,theta,phi,
     &     cf,itheta,iphi)
      
      if (cf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'angular sector could not be identified'
         write(*,*) 'us=',(us(i),i=1,dim)
         write(*,*) 'Ntheta=',Ntheta
         write(*,*) 'Nphi=',Nphi
         stop
      endif

      return
      end





      subroutine ground_normal(dim,P,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,n)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the normal to the ground surface at
c     a given position
c
c     Inputs:
c       + dim: dimension of vectors
c       + P: (r,theta,phi) position the normal has to be computed for
c       + theta1,theta2,phi1,phi2: latitude and longitude limits
c       + ca,cb,cc: a, b and c coefficients for each quarter
c
c     Output:
c       + n: (x,y,z) normal to the ground at position "P"
c

c     inputs
      integer dim
      double precision P(1:Nmat_mx)
      double precision theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     outputs
      double precision n(1:Nmat_mx)
c     temp
      integer i
      double precision dtheta,dphi
      double precision pos1(1:2)
      double precision pos2(1:2)
      double precision xP0(1:Nmat_mx)
      double precision xP1(1:Nmat_mx)
      double precision xP2(1:Nmat_mx)
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision x1(1:Nmat_mx)
      double precision x2(1:Nmat_mx)
      double precision length
      integer strlen
      character*100 label
      label='subroutine ground_normal'

      dtheta=1.0D-4 ! rad
      dphi=dtheta

      call spher2cart(label,1,P(1),P(2),P(3),xP0(1),xP0(2),xP0(3))

      P1(2)=P(2)
      P1(3)=P(3)+dphi
      pos1(1)=P1(2)
      pos1(2)=P1(3)
      call ground_radius(pos1,theta1,theta2,
     &     phi1,phi2,ca,cb,cc,P1(1))
      P2(2)=P(2)+dtheta
      P2(3)=P(3)
      pos2(1)=P2(2)
      pos2(2)=P2(3)
      call ground_radius(pos2,theta1,theta2,
     &     phi1,phi2,ca,cb,cc,P2(1))

c     x1 is the vector (cartesian coordinates) that is obtained between
c     position "P" and a position "P1" obtained by shifting position "P"
c     by a small longitude angle "dphi"
      call spher2cart(label,0,P1(1),P1(2),P1(3),xP1(1),xP1(2),xP1(3))
      call substract_vectors(dim,xP1,xP0,x1)
      
c     x2 is the vector (cartesian coordinates) that is obtained between
c     position "P" and a position "P2" obtained by shifting position "P"
c     by a small latitude angle "dtheta"
      call spher2cart(label,0,P2(1),P2(2),P2(3),xP2(1),xP2(2),xP2(3))
      call substract_vectors(dim,xP2,xP0,x2)

c     computation of n, the inside normal to the ground at position "P"
      call cross_product(dim,x1,x2,n)

c     Debug
      call vector_length(dim,n,length)
      if (dabs(length-1.0D+0).gt.1.0D-2) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'n=',(n(i),i=1,dim)
         write(*,*) 'length=',length
         stop
      endif
c     Debug

      return
      end




      subroutine extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &     ca,cb,cc)
      implicit none
c
c     Purpose: to extract the four coefficients that define the 4 ground surfaces
c     for a given column
c
c     Inputs:
c       + itheta,iphi: index of the column
c       + coeff_a,coeff_b,coeff_c: a, b and c coefficients for the whole geometry
c     
c     Outputs:
c       + ca,cb,cc: a, b and c coefficients for the column only
c
      include 'max.inc'
c     input
      integer itheta,iphi
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
c     output
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     temp
      integer i


      do i=1,4
         ca(i)=coeff_a(itheta,iphi,i)
         cb(i)=coeff_b(itheta,iphi,i)
         cc(i)=coeff_c(itheta,iphi,i)
      enddo ! i

      return
      end



      subroutine ground_radius(P,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,radius)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the radius of the planet at a given location
c
c     Inputs:
c       + P: (theta,phi) position the radius has to be computed for
c       + theta1,theta2,phi2,phi2: latitude and longitude limits of the sector
c       + coeff_a,coeff_b,coeff_c: a, b and c coefficients for each quarter
c
c     Outputs:
c       + radius: radius at position (theta,phi)
c

c     inputs
      double precision P(1:2)
      double precision theta1,theta2
      double precision phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     outputs
      double precision radius
c     temp
      integer quarter
      double precision a,b,c
      integer strlen,i
      character*100 label
      label='subroutine ground_radius'

      call identify_quarter(P,theta1,theta2,phi1,phi2,quarter)
      call quarter_radius(P,ca(quarter),cb(quarter),cc(quarter),radius)

      return
      end


      subroutine quarter_radius(P,a,b,c,radius)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the radius for a given quarter
c
c     Inputs:
c       + P: (theta,phi) position the radius has to be computed for
c       + a, b and c: coefficients for the quarter
c
c     Output:
c       + radius: radius for the quarter
c

c     input
      double precision P(1:2)
      double precision a,b,c
c     output
      double precision radius
c     temp
      integer strlen
      character*100 label
      label='subroutine quarter_radius'

      radius=a*P(1)+b*P(2)+c
c     Debug
      if (radius.lt.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'radius=',radius
         write(*,*) 'a=',a
         write(*,*) 'b=',b
         write(*,*) 'c=',c
         write(*,*) 'P(1)=',P(1)
         write(*,*) 'P(2)=',P(2)
         stop
      endif
c     Debug

      return
      end

      



      subroutine identify_quarter(P,theta1,theta2,phi1,phi2,quarter)
      implicit none
      include 'max.inc'
c
c     Purpose: to provide the index of the quarter position "P" is in
c
c     Inputs:
c       + P: (theta,phi) latitude and longitude coordinates of the position
c       + theta1,theta2,phi2,phi2: latitude and longitude limits of the sector
c
c     Output:
c       + quarter: index of the quarter "P" is in
c

c     input
      double precision P(1:2)
      double precision theta1,theta2
      double precision phi1,phi2
c     output
      integer quarter
c     temp
      double precision theta_d1,theta_d2
      integer ud1,ud2
      integer strlen
      character*100 label
      label='subroutine identify_quarter'

      theta_d1=theta2-
     &     (theta2-theta1)
     &     /(phi2-phi1)
     &     *(P(2)-phi1)
      theta_d2=theta1+
     &     (theta2-theta1)
     &     /(phi2-phi1)
     &     *(P(2)-phi1)
      if (P(1).ge.theta_d1) then
         if (P(1).ge.theta_d2) then
            quarter=1
         else
            quarter=4
         endif
      else
         if (P(1).ge.theta_d2) then
            quarter=2
         else
            quarter=3
         endif
      endif

      return
      end




      subroutine surface_coeffs(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c)
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute coefficients "a", "b" and "c" for each quarter in the
c     (r,theta,phi) space
c
c            -> phi
c     
c     -------------------- theta(itheta)
c     |\                /|
c     | \              / |
c     |  \    q1      /  |
c     |   \          /   |
c     |    \        /    |
c     |     \      /     |
c     |      \    /      |
c     |       \  /       |
c     | q2     \/   q4   |      ^
c     |        /\        |      | theta
c     |       /  \       |
c     |      /    \      |
c     |     /      \     |
c     |    /        \    |
c     |   /   q3     \   |
c     |  /            \  |
c     | /              \ |
c     |/                \|
c     -------------------  theta(itheta-1)
c
c     |                  |
c    phi(iphi-(1))      phi(iphi)
c
c     Inputs:
c       + dim: dimension of vectors
c       + Nr,Ntheta,Nphi: number of cells
c       + r,theta,phi: limits of spherical cells
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c
c     Outputs:
c       + coeff_a, coeff_b, coeff_c
c

c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
c     outputs
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
c     temp
      integer itheta,iphi,quarter
      integer itheta_prev,itheta_next
      integer iphi_prev,iphi_next
      double precision thetac,phic
      double precision mat(1:Nmat_mx,1:Nmat_mx)
      double precision y(1:Nmat_mx)
      double precision x(1:Nmat_mx)
      double precision coeff_max
      integer valid,i,j
      integer node,raf,q,qmin,qmax
      double precision rn
      double precision P(1:2)
      double precision rnq(1:8)
      double precision a,b,c
      double precision err_max
      integer strlen
      character*100 label
c     Debug
      double precision sum,pos(1:2)
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision r_ground,radius
      integer it_test,ip_test
c     Debug
 20   format(I3,I3,4f18.9)
      label='subroutine surface_coeffs'

      coeff_max=1.0D+5

      do i=1,3
         mat(i,3)=1.0D+0
      enddo ! i

c     Debug
c      write(*,*) 'r_north_pole=',r_north_pole
c     Debug

      do itheta=1,Ntheta
         do iphi=1,Nphi
            thetac=(theta(itheta-1)+theta(itheta))/2.0D+0
            phic=(phi(iphi-1)+phi(iphi))/2.0D+0
            mat(1,1)=thetac
            mat(1,2)=phic
            y(1)=r(0,itheta,iphi)
c     quarter 1
            mat(2,1)=theta(itheta)
            mat(2,2)=phi(iphi-1)
            mat(3,1)=theta(itheta)
            mat(3,2)=phi(iphi)
            if (itheta.eq.1) then
               y(2)=r_nodes(itheta+1,iphi,2)
               y(3)=r_nodes(itheta+1,iphi,3)
            else if (itheta.eq.Ntheta) then
               y(2)=r_north_pole
               y(3)=r_north_pole
            else
               y(2)=r_nodes(itheta,iphi,1)
               y(3)=r_nodes(itheta,iphi,4)
            endif
            call linear_system(0,dim,mat,y,valid,x)
            coeff_a(itheta,iphi,1)=x(1)
            coeff_b(itheta,iphi,1)=x(2)
            coeff_c(itheta,iphi,1)=x(3)
c     quarter 2
            mat(2,1)=theta(itheta-1)
            mat(2,2)=phi(iphi-1)
            mat(3,1)=theta(itheta)
            mat(3,2)=phi(iphi-1)
            if (itheta.eq.1) then
               y(2)=r_south_pole
               y(3)=r_nodes(itheta+1,iphi,2)
            else if (itheta.eq.Ntheta) then
               y(2)=r_nodes(itheta-1,iphi,1)
               y(3)=r_north_pole
            else
               y(2)=r_nodes(itheta,iphi,2)
               y(3)=r_nodes(itheta,iphi,1)
            endif
            call linear_system(0,dim,mat,y,valid,x)
            coeff_a(itheta,iphi,2)=x(1)
            coeff_b(itheta,iphi,2)=x(2)
            coeff_c(itheta,iphi,2)=x(3)
c     quarter 3
            mat(2,1)=theta(itheta-1)
            mat(2,2)=phi(iphi)
            mat(3,1)=theta(itheta-1)
            mat(3,2)=phi(iphi-1)
            if (itheta.eq.1) then
               y(2)=r_south_pole
               y(3)=r_south_pole
            else if (itheta.eq.Ntheta) then
               y(2)=r_nodes(itheta-1,iphi,4)
               y(3)=r_nodes(itheta-1,iphi,1)
            else
               y(2)=r_nodes(itheta,iphi,3)
               y(3)=r_nodes(itheta,iphi,2)
            endif
            call linear_system(0,dim,mat,y,valid,x)
            coeff_a(itheta,iphi,3)=x(1)
            coeff_b(itheta,iphi,3)=x(2)
            coeff_c(itheta,iphi,3)=x(3)
c     quarter 4
            mat(2,1)=theta(itheta)
            mat(2,2)=phi(iphi)
            mat(3,1)=theta(itheta-1)
            mat(3,2)=phi(iphi)
            if (itheta.eq.1) then
               y(2)=r_nodes(itheta+1,iphi,3)
               y(3)=r_south_pole
            else if (itheta.eq.Ntheta) then
               y(2)=r_north_pole
               y(3)=r_nodes(itheta-1,iphi,4)
            else
               y(2)=r_nodes(itheta,iphi,4)
               y(3)=r_nodes(itheta,iphi,3)
            endif
            call linear_system(0,dim,mat,y,valid,x)
            coeff_a(itheta,iphi,4)=x(1)
            coeff_b(itheta,iphi,4)=x(2)
            coeff_c(itheta,iphi,4)=x(3)
c     Debug
c            if ((itheta.eq.9).and.(iphi.eq.40)) then
c               do i=1,3
c                  write(*,*) 'x(',i,')=',x(i)
c               enddo
c            endif
c     Debug
c     Errors due to linear systems that have to solution
            do quarter=1,4
               if (dabs(coeff_a(itheta,iphi,quarter)).
     &              gt.coeff_max) then
                  coeff_a(itheta,iphi,quarter)=0.0D+0
               endif
               if (dabs(coeff_b(itheta,iphi,quarter)).
     &              gt.coeff_max) then
                  coeff_b(itheta,iphi,quarter)=0.0D+0
               endif
            enddo ! quarter
c     Debug
         enddo ! iphi
      enddo ! itheta

c     verification: the altitude of each node may be retrieved using coefficients
c     of the 8 surrouding quarters
c     Debug
      write(*,*) 'Checking nodes radiuses...'
c     Debug
      err_max=1.0D+0
      do itheta=2,Ntheta-1
         do iphi=1,Nphi
            call next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           itheta_next,raf)
            call prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           itheta_prev,raf)
            call next_phi_cell(Nphi,iphi,iphi_next)
            call prev_phi_cell(Nphi,iphi,iphi_prev)
            do node=1,4
               rn=r_nodes(itheta,iphi,node)
c     node 1
               if (node.eq.1) then
                  P(1)=theta(itheta)
                  P(2)=phi(iphi-1)
                  a=coeff_a(itheta,iphi_prev,1)
                  b=coeff_b(itheta,iphi_prev,1)
                  c=coeff_c(itheta,iphi_prev,1)
                  call quarter_radius(P,a,b,c,rnq(7))
                  a=coeff_a(itheta,iphi_prev,4)
                  b=coeff_b(itheta,iphi_prev,4)
                  c=coeff_c(itheta,iphi_prev,4)
                  call quarter_radius(P,a,b,c,rnq(8))
                  a=coeff_a(itheta,iphi,2)
                  b=coeff_b(itheta,iphi,2)
                  c=coeff_c(itheta,iphi,2)
                  call quarter_radius(P,a,b,c,rnq(1))
                  a=coeff_a(itheta,iphi,1)
                  b=coeff_b(itheta,iphi,1)
                  c=coeff_c(itheta,iphi,1)
                  call quarter_radius(P,a,b,c,rnq(2))
                  a=coeff_a(itheta_next,iphi,3)
                  b=coeff_b(itheta_next,iphi,3)
                  c=coeff_c(itheta_next,iphi,3)
                  call quarter_radius(P,a,b,c,rnq(3))
                  a=coeff_a(itheta_next,iphi,2)
                  b=coeff_b(itheta_next,iphi,2)
                  c=coeff_c(itheta_next,iphi,2)
                  call quarter_radius(P,a,b,c,rnq(4))
                  a=coeff_a(itheta_next,iphi_prev,4)
                  b=coeff_b(itheta_next,iphi_prev,4)
                  c=coeff_c(itheta_next,iphi_prev,4)
                  call quarter_radius(P,a,b,c,rnq(5))
                  a=coeff_a(itheta_next,iphi_prev,3)
                  b=coeff_b(itheta_next,iphi_prev,3)
                  c=coeff_c(itheta_next,iphi_prev,3)
                  call quarter_radius(P,a,b,c,rnq(6))
c     node 2
               else if (node.eq.2) then
                  P(1)=theta(itheta-1)
                  P(2)=phi(iphi-1)
                  a=coeff_a(itheta_prev,iphi_prev,1)
                  b=coeff_b(itheta_prev,iphi_prev,1)
                  c=coeff_c(itheta_prev,iphi_prev,1)
                  call quarter_radius(P,a,b,c,rnq(7))
                  a=coeff_a(itheta_prev,iphi_prev,4)
                  b=coeff_b(itheta_prev,iphi_prev,4)
                  c=coeff_c(itheta_prev,iphi_prev,4)
                  call quarter_radius(P,a,b,c,rnq(8))
                  a=coeff_a(itheta_prev,iphi,2)
                  b=coeff_b(itheta_prev,iphi,2)
                  c=coeff_c(itheta_prev,iphi,2)
                  call quarter_radius(P,a,b,c,rnq(1))
                  a=coeff_a(itheta_prev,iphi,1)
                  b=coeff_b(itheta_prev,iphi,1)
                  c=coeff_c(itheta_prev,iphi,1)
                  call quarter_radius(P,a,b,c,rnq(2))
                  a=coeff_a(itheta,iphi,3)
                  b=coeff_b(itheta,iphi,3)
                  c=coeff_c(itheta,iphi,3)
                  call quarter_radius(P,a,b,c,rnq(3))
                  a=coeff_a(itheta,iphi,2)
                  b=coeff_b(itheta,iphi,2)
                  c=coeff_c(itheta,iphi,2)
                  call quarter_radius(P,a,b,c,rnq(4))
                  a=coeff_a(itheta,iphi_prev,4)
                  b=coeff_b(itheta,iphi_prev,4)
                  c=coeff_c(itheta,iphi_prev,4)
                  call quarter_radius(P,a,b,c,rnq(5))
                  a=coeff_a(itheta,iphi_prev,3)
                  b=coeff_b(itheta,iphi_prev,3)
                  c=coeff_c(itheta,iphi_prev,3)
                  call quarter_radius(P,a,b,c,rnq(6))
c     node 3
               else if (node.eq.3) then
                  P(1)=theta(itheta-1)
                  P(2)=phi(iphi)
                  a=coeff_a(itheta_prev,iphi,1)
                  b=coeff_b(itheta_prev,iphi,1)
                  c=coeff_c(itheta_prev,iphi,1)
                  call quarter_radius(P,a,b,c,rnq(7))
                  a=coeff_a(itheta_prev,iphi,4)
                  b=coeff_b(itheta_prev,iphi,4)
                  c=coeff_c(itheta_prev,iphi,4)
                  call quarter_radius(P,a,b,c,rnq(8))
                  a=coeff_a(itheta_prev,iphi_next,2)
                  b=coeff_b(itheta_prev,iphi_next,2)
                  c=coeff_c(itheta_prev,iphi_next,2)
                  call quarter_radius(P,a,b,c,rnq(1))
                  a=coeff_a(itheta_prev,iphi_next,1)
                  b=coeff_b(itheta_prev,iphi_next,1)
                  c=coeff_c(itheta_prev,iphi_next,1)
                  call quarter_radius(P,a,b,c,rnq(2))
                  a=coeff_a(itheta,iphi_next,3)
                  b=coeff_b(itheta,iphi_next,3)
                  c=coeff_c(itheta,iphi_next,3)
                  call quarter_radius(P,a,b,c,rnq(3))
                  a=coeff_a(itheta,iphi_next,2)
                  b=coeff_b(itheta,iphi_next,2)
                  c=coeff_c(itheta,iphi_next,2)
                  call quarter_radius(P,a,b,c,rnq(4))
                  a=coeff_a(itheta,iphi,4)
                  b=coeff_b(itheta,iphi,4)
                  c=coeff_c(itheta,iphi,4)
                  call quarter_radius(P,a,b,c,rnq(5))
                  a=coeff_a(itheta,iphi,3)
                  b=coeff_b(itheta,iphi,3)
                  c=coeff_c(itheta,iphi,3)
                  call quarter_radius(P,a,b,c,rnq(6))
c     node 4
               else if (node.eq.4) then
                  P(1)=theta(itheta)
                  P(2)=phi(iphi)
                  a=coeff_a(itheta,iphi,1)
                  b=coeff_b(itheta,iphi,1)
                  c=coeff_c(itheta,iphi,1)
                  call quarter_radius(P,a,b,c,rnq(7))
                  a=coeff_a(itheta,iphi,4)
                  b=coeff_b(itheta,iphi,4)
                  c=coeff_c(itheta,iphi,4)
                  call quarter_radius(P,a,b,c,rnq(8))
                  a=coeff_a(itheta,iphi_next,2)
                  b=coeff_b(itheta,iphi_next,2)
                  c=coeff_c(itheta,iphi_next,2)
                  call quarter_radius(P,a,b,c,rnq(1))
                  a=coeff_a(itheta,iphi_next,1)
                  b=coeff_b(itheta,iphi_next,1)
                  c=coeff_c(itheta,iphi_next,1)
                  call quarter_radius(P,a,b,c,rnq(2))
                  a=coeff_a(itheta_next,iphi_next,3)
                  b=coeff_b(itheta_next,iphi_next,3)
                  c=coeff_c(itheta_next,iphi_next,3)
                  call quarter_radius(P,a,b,c,rnq(3))
                  a=coeff_a(itheta_next,iphi_next,2)
                  b=coeff_b(itheta_next,iphi_next,2)
                  c=coeff_c(itheta_next,iphi_next,2)
                  call quarter_radius(P,a,b,c,rnq(4))
                  a=coeff_a(itheta_next,iphi,4)
                  b=coeff_b(itheta_next,iphi,4)
                  c=coeff_c(itheta_next,iphi,4)
                  call quarter_radius(P,a,b,c,rnq(5))
                  a=coeff_a(itheta_next,iphi,3)
                  b=coeff_b(itheta_next,iphi,3)
                  c=coeff_c(itheta_next,iphi,3)
                  call quarter_radius(P,a,b,c,rnq(6))
               endif
               if (iphi.eq.1) then
                  qmin=1
                  qmax=4
               else if (iphi.eq.Nphi) then
                  qmin=5
                  qmax=8
               else
                  qmin=1
                  qmax=8
               endif
               do q=qmin,qmax
                  if (dabs(rn-rnq(q)).gt.err_max) then
                     write(*,*) 'Error from ',
     &                    label(1:strlen(label)),' :'
                     write(*,*) 'r_nodes(',itheta,',',iphi,
     &                    ',',node,')=',r_nodes(itheta,iphi,node)
                     write(*,*) 'rnq(',q,')=',rnq(q)
c     Debug
c                     a=coeff_a(itheta,iphi_next,1)
c                     b=coeff_b(itheta,iphi_next,1)
c                     c=coeff_c(itheta,iphi_next,1)
c                     P(1)=theta(itheta)
c                     P(2)=phi(iphi)
c                     call quarter_radius(P,a,b,c,rnq(2))
c                     write(*,*) 'itheta=',itheta
c                     write(*,*) 'iphi_next=',iphi_next
c                     write(*,*) 'a=',a
c                     write(*,*) 'b=',b
c                     write(*,*) 'c=',c
c                     write(*,*) 'P(1)=',P(1)
c                     write(*,*) 'P(2)=',P(2)
c                     write(*,*) 'rnq(2)=',rnq(2)
c
c                     write(*,*) 'solving again'
c                     do i=1,3
c                        mat(i,3)=1.0D+0
c                     enddo      ! i
c                     it_test=9
c                     ip_test=2
c                     thetac=(theta(it_test-1)+theta(it_test))/2.0D+0
c                     phic=(phi(ip_test-1)+phi(ip_test))/2.0D+0
c                     mat(1,1)=thetac
c                     mat(1,2)=phic
c                     y(1)=r(0,it_test,ip_test)
c                     mat(2,1)=theta(it_test)
c                     mat(2,2)=phi(ip_test-1)
c                     mat(3,1)=theta(it_test)
c                     mat(3,2)=phi(ip_test)
c                     y(2)=r_nodes(it_test,ip_test,1)
c                     y(3)=r_nodes(it_test,ip_test,4)
c                     write(*,*) 'mat='
c                     do i=1,3
c                        write(*,*) (mat(i,j),j=1,3),y(i)
c                     enddo
c                     call linear_system(1,dim,mat,y,valid,x)
c                     do i=1,3
c                        write(*,*) 'x(',i,')=',x(i)
c                     enddo
c                     do i=1,3
c                        sum=0.0D+0
c                        do j=1,3
c                           sum=sum+mat(i,j)*x(j)
c                        enddo
c                        write(*,*) 'sum(',i,')=',sum
c                     enddo
c                     P(1)=theta(it_test)
c                     P(2)=phi(ip_test-1)
c                     call quarter_radius(P,x(1),x(2),x(3),radius)
c                     write(*,*) 'theta=',P(1)
c                     write(*,*) 'phi=',P(2)
c                     write(*,*) 'r=',radius
c     Debug
                     stop
                  endif
               enddo ! q
            enddo ! node
         enddo ! iphi
      enddo ! itheta
c     r_south_pole
      P(1)=theta(0)
      P(2)=phi(0)
      itheta=1
      rn=r_south_pole
      do iphi=1,Nphi
         a=coeff_a(itheta,iphi,2)
         b=coeff_b(itheta,iphi,2)
         c=coeff_c(itheta,iphi,2)
         call quarter_radius(P,a,b,c,rnq(1))
c     Debug
c         write(*,*) 'a=',a
c         write(*,*) 'b=',b
c         write(*,*) 'c=',c
c         write(*,*) 'rnq(',1,')=',rnq(1)
c     Debug
         a=coeff_a(itheta,iphi,4)
         b=coeff_b(itheta,iphi,4)
         c=coeff_c(itheta,iphi,4)
         call quarter_radius(P,a,b,c,rnq(2))
c     Debug
c         write(*,*) 'a=',a
c         write(*,*) 'b=',b
c         write(*,*) 'c=',c
c         write(*,*) 'rnq(',2,')=',rnq(2)
c     Debug
         do q=1,2
            if (dabs(rn-rnq(q)).gt.err_max) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r_south_pole=',r_south_pole
               write(*,*) 'rnq(',q,')=',rnq(q)
               stop
            endif
         enddo
      enddo ! iphi
c     r_north_pole
      P(1)=theta(Ntheta)
      P(2)=phi(0)
      itheta=1
      rn=r_north_pole
      do iphi=1,Nphi
         a=coeff_a(itheta,iphi,2)
         b=coeff_b(itheta,iphi,2)
         c=coeff_c(itheta,iphi,2)
         call quarter_radius(P,a,b,c,rnq(1))
c     Debug
c         write(*,*) 'a=',a
c         write(*,*) 'b=',b
c         write(*,*) 'c=',c
c         write(*,*) 'rnq(',1,')=',rnq(1)
c     Debug
         a=coeff_a(itheta,iphi,4)
         b=coeff_b(itheta,iphi,4)
         c=coeff_c(itheta,iphi,4)
         call quarter_radius(P,a,b,c,rnq(2))
c     Debug
c         write(*,*) 'a=',a
c         write(*,*) 'b=',b
c         write(*,*) 'c=',c
c         write(*,*) 'rnq(',2,')=',rnq(2)
c     Debug
         do q=1,2
            if (dabs(rn-rnq(q)).gt.err_max) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r_north_pole=',r_north_pole
               write(*,*) 'rnq(',q,')=',rnq(q)
               stop
            endif
         enddo
      enddo ! iphi
c     Debug
      write(*,*) '...done'
c     Debug

c     Debug
      open(11,file='./logs/quarter_a_coeffs.log')
      open(12,file='./logs/quarter_b_coeffs.log')
      open(13,file='./logs/quarter_c_coeffs.log')
      write(11,*) 'itheta / iphi / coeff_a(itheta,iphi,j),j=1,4'
      write(12,*) 'itheta / iphi / coeff_b(itheta,iphi,j),j=1,4'
      write(13,*) 'itheta / iphi / coeff_c(itheta,iphi,j),j=1,4'
      do itheta=1,Ntheta
         do iphi=1,Nphi
            write(11,20) itheta,iphi,(coeff_a(itheta,iphi,j),j=1,4)
            write(12,20) itheta,iphi,(coeff_b(itheta,iphi,j),j=1,4)
            write(13,20) itheta,iphi,(coeff_c(itheta,iphi,j),j=1,4)
         enddo ! iphi
      enddo ! itheta
      close(11)
      close(12)
      close(13)
c     Debug

      return
      end





      subroutine nodes_ground_altitude(dim,Nr,Ntheta,Nphi,
     &     r,theta,phi,r_nodes,r_south_pole,r_north_pole)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute ground altitudes at grid nodes, from ground altitudes
c     taken at center of each (theta,phi) angular sector
c
c     Inputs:
c       + dim: dimension of vectors
c       + Nr,Ntheta,Nphi: number of cells
c       + r,theta,phi: limits of spherical cells
c
c     Outputs:
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c

c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
c     outputs
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
c     temp
      integer node
      integer nelem,d
      integer itheta,iphi
      integer ithetap,iphip
      integer ithetam,iphim
      integer raf,j
      double precision Pi(1:2,1:Nobs_mx)
      double precision fi(1:Nobs_mx)
      double precision P(1:2)
      integer strlen
      character*100 label
 20   format(I3,I3,4f18.9)
      label='subroutine nodes_ground_altitude'
      nelem=4
      d=1

      if ((Ntheta.eq.1).and.(Nphi.eq.1)) then
         do node=1,4
            r_nodes(1,1,node)=r(0,1,1)
         enddo
         r_south_pole=r(0,1,1)
         r_north_pole=r(0,1,1)
         goto 321
      endif

      if (Ntheta.le.2) then
         goto 123
      endif
      if (Nobs_mx.lt.nelem) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'nelem=',nelem
         write(*,*) 'while Nobs_mx=',Nobs_mx
         stop
      endif

      do itheta=2,Ntheta-1
         do iphi=1,Nphi
            call prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           ithetam,raf)
            call next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           ithetap,raf)
            call prev_phi_cell(Nphi,iphi,iphim)
            call next_phi_cell(Nphi,iphi,iphip)
c     node 1
            call set_Pi_fi(r,theta,phi,ithetap,itheta,iphi,iphim,
     &           Pi,fi)
            P(1)=theta(itheta)
            P(2)=phi(iphi-1)
            call lin_interp2d_spher(dim,nelem,Pi,fi,P,d,
     &           r_nodes(itheta,iphi,1))
c     node 2
            call set_Pi_fi(r,theta,phi,itheta,ithetam,iphi,iphim,
     &           Pi,fi)
            P(1)=theta(itheta-1)
            P(2)=phi(iphi-1)
            call lin_interp2d_spher(dim,nelem,Pi,fi,P,d,
     &           r_nodes(itheta,iphi,2))
c     node 3
            call set_Pi_fi(r,theta,phi,itheta,ithetam,iphip,iphi,
     &           Pi,fi)
            P(1)=theta(itheta-1)
            P(2)=phi(iphi)
            call lin_interp2d_spher(dim,nelem,Pi,fi,P,d,
     &           r_nodes(itheta,iphi,3))
c     node 4
            call set_Pi_fi(r,theta,phi,ithetap,itheta,iphip,iphi,
     &           Pi,fi)
            P(1)=theta(itheta)
            P(2)=phi(iphi)
            call lin_interp2d_spher(dim,nelem,Pi,fi,P,d,
     &           r_nodes(itheta,iphi,4))
         enddo ! iphi
      enddo ! itheta
 123  continue
      
c     altitudes of the poles
      call pole_ground_altitude(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     -1,r_south_pole)
      call pole_ground_altitude(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     1,r_north_pole)

 321  continue
c     Debug
      open(10,file='./logs/r_nodes.log')
      write(10,*) 'itheta / iphi / r_nodes(itheta,iphi,j),j=1,4'
      do itheta=2,Ntheta-1
         do iphi=1,Nphi
            write(10,20) itheta,iphi,(r_nodes(itheta,iphi,j),j=1,4)
         enddo ! iphi
      enddo ! itheta
      write(10,*)
      write(10,*) 'r south pole:',r_south_pole
      write(10,*) 'r north pole:',r_north_pole
      close(10)
c     Debug

      return
      end



      subroutine pole_ground_altitude(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     pole,r_pole)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute ground altitude (radius) at a given pole
c
c     Inputs:
c       + dim: dimension of vectors
c       + Nr,Ntheta,Nphi: number of cells
c       + r,theta,phi: limits of spherical cells
c       + pole:
c         - pole=-1: south pole
c         - pole=1: north pole
c
c     Outputs:
c       + r_pole: radius at pole
c

c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      integer pole
c     output
      double precision r_pole
c     temp
      integer itheta,iphi
      integer nelem,i
      double precision Pi(1:2,1:Nobs_mx)
      double precision fi(1:Nobs_mx)
      double precision P(1:2),d
      integer strlen
      character*100 label
      label='subroutine pole_ground_altitude'
      nelem=Nphi
      d=1

      if (Nobs_mx.lt.nelem) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'nelem=',nelem
         write(*,*) 'while Nobs_mx=',Nobs_mx
         stop
      endif

      if (pole.eq.-1) then
         itheta=1
      else if (pole.eq.1) then
         itheta=Ntheta
      else
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'bad input argument pole=',pole
         stop
      endif

      do i=1,Nphi
         Pi(1,i)=(theta(itheta-1)/theta(itheta))/2.0D+0
         Pi(2,i)=(phi(i-1)+phi(i))/2.0D+0
         fi(i)=r(0,itheta,i)
      enddo ! iphi
      call lin_interp2d_spher(dim,nelem,Pi,fi,P,d,r_pole)

      return
      end




      subroutine set_Pi_fi(r,theta,phi,ithetau,ithetad,iphiu,iphid,
     &     Pi,fi)
      implicit none
      include 'max.inc'
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      integer ithetau,ithetad,iphiu,iphid
      double precision Pi(1:2,1:Nobs_mx)
      double precision fi(1:Nobs_mx)

      Pi(1,1)=(theta(ithetau-1)+theta(ithetau))/2.0D+0
      Pi(2,1)=(phi(iphid-1)+phi(iphid))/2.0D+0
      Pi(1,2)=(theta(ithetad-1)+theta(ithetad))/2.0D+0
      Pi(2,2)=(phi(iphid-1)+phi(iphid))/2.0D+0
      Pi(1,3)=(theta(ithetad-1)+theta(ithetad))/2.0D+0
      Pi(2,3)=(phi(iphiu-1)+phi(iphiu))/2.0D+0
      Pi(1,4)=(theta(ithetau-1)+theta(ithetau))/2.0D+0
      Pi(2,4)=(phi(iphiu-1)+phi(iphiu))/2.0D+0
      fi(1)=r(0,ithetau,iphid)
      fi(2)=r(0,ithetad,iphid)
      fi(3)=r(0,ithetad,iphiu)
      fi(4)=r(0,ithetau,iphiu)

      return
      end




      subroutine new_cell(db,dim,ioe,edge,Pstart,u_in,Nr,Ntheta,Nphi,
     &     r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,coeff_a,coeff_b,coeff_c,
     &     ct,surface,ir,itheta,iphi,
     &     P_out,ct_new,surface_new,new_ir,new_itheta,new_iphi)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to compute the cell the particle is going into,
c     from the index of the cell the particle comes from
c     
c     Inputs:
c       + dim: dimension of vectors
c       + ioe: 1 if the particle exits its current cell from an edge
c       + Pstart: (r,theta,phi) spherical coordinates of the entry position
c       + u_in: propagation direction at input
c       + edge: index of the edge when ioe=1
c       + Nr,Ntheta,Nphi: number of cells
c       + r,theta,phi: limits of each cell
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + coeff_a, coeff_b, coeff_c: a, b and c coefficients for each quarter
c       + ct: cell type
c         - ct=1: the particle has just been emitted by a surface
c         - ct=2: the particle comes from another cell (volume)
c       + surface: surface type
c         - if the particle has been emitted by a surface (ct=1), then "surface"
c          must take value 1 if the particle has been emitted from ground, or
c          value 2 if the particle comes from space
c         - if the particle comes from another cell (ct=2), then "surface" is the
c          type of surface (1-6) the particle exits the cell from.
c      + ir,itheta,iphi: index of the cell the particle comes from (ct=2)
c
c     Outputs:
c       + P_out: updated position of the particle
c       + ct_new: new cell type (1,2)
c       + surface_new: surface type the particle goes through
c         - ct_new=1 (the particle goes into a surface): surface_new
c          takes value 1 (ground) or 2 (space)
c         - ct_new=2 (the particle goes into a volumic cell): surface_new
c          takes value 1-6, type of surface the particle enters the cell from
c       + new_ir,new_itheta,new_iphi: index of the cell the particle
c        goes into (ct_new=2)
c

c     input
      integer db,dim,ioe,edge
      double precision Pstart(1:Nmat_mx)
      double precision u_in(1:Nmat_mx)
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
      integer ct,surface
      integer ir,itheta,iphi
c     output
      double precision P_out(1:Nmat_mx)
      integer ct_new,surface_new
      integer new_ir,new_itheta,new_iphi
c     temp
      integer ons
      double precision P(1:Nmat_mx)
      double precision n_surf(1:Nmat_mx)
      double precision pos(1:2),r_ground
      integer iphi_mod
      double precision P1(1:Nmat_mx)
      integer i,ir1,itheta1,iphi1,cellf,columnf
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer strlen
      character*100 label
      label='subroutine new_cell'

c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :in'
      endif
c     Debug
      
c     Debug
      if ((surface.lt.1).or.(surface.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'Bad input parameter:'
         write(*,*) 'surface=',surface
         write(*,*) 'position 1'
         write(*,*) 'db=',db
         write(*,*) 'dim=',dim
         write(*,*) 'ioe=',ioe
         write(*,*) 'edge=',edge
         write(*,*) 'ct=',ct
         stop
      endif
c     Debug

      call copy_vector(dim,Pstart,P)

      if (ioe.eq.1) then
         call jump_to(dim,Pstart,u_in,1.0D+1,P1)
c     Debug
         if (db.eq.1) then
            write(*,*) 'Pstart=',(Pstart(i),i=1,dim)
            write(*,*) 'P1=',(P1(i),i=1,dim)
         endif
c     Debug
         call identify_cell(dim,P1,Nr,Ntheta,Nphi,r,theta,phi,
     &        coeff_a,coeff_b,coeff_c,
     &        r_nodes,r_south_pole,r_north_pole,
     &        cellf,ir1,itheta1,iphi1)
         if ((iphi1.eq.Nphi).and.(P(3).eq.0.0D+0)) then
            P(3)=2.0D+0*pi
         else if ((iphi1.eq.1).and.(P(3).eq.2.0D+0*pi)) then
            P(3)=0.0D+0
         endif
c     Debug
         if (db.eq.1) then
            write(*,*) 'cellf=',cellf
         endif
c     Debug
         if (cellf.eq.1) then
            new_ir=ir1
            new_itheta=itheta1
            new_iphi=iphi1
            ct_new=2
c     find index of "surface_new"
            call cell_surface(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,
     &           coeff_a,coeff_b,coeff_c,P,new_ir,new_itheta,new_iphi,
     &           ons,surface_new,n_surf)
            if (ons.eq.0) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'ioe=',ioe
               write(*,*) 'ons=',ons
               write(*,*) 'P=',(P(i),i=1,dim)
               write(*,*) 'new_ir=',new_ir
               write(*,*) r(new_ir-1,new_itheta,new_iphi),
     &              r(new_ir,new_itheta,new_iphi)
               write(*,*) 'new_itheta=',new_itheta
               write(*,*) theta(new_itheta-1),theta(new_itheta)
               write(*,*) 'new_iphi=',new_iphi
               write(*,*) phi(new_iphi-1),phi(new_iphi)
               stop
            endif
c     Debug
            if (db.eq.1) then
               write(*,*) 'new_ir=',new_ir
               write(*,*) 'new_itheta=',new_itheta
               write(*,*) 'new_iphi=',new_iphi
               write(*,*) 'surface=',surface,
     &              ' surface_new=',surface_new
            endif
c     Debug
            goto 123
         else ! if cellf=0
            call identify_column(P1,Ntheta,Nphi,theta,phi,
     &           columnf,itheta1,iphi1)
c     Debug
            if (db.eq.1) then
               write(*,*) 'columnf=',columnf
            endif
c     Debug
            if (columnf.eq.0) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'columnf=',columnf
               stop
            endif
            new_itheta=itheta1
            new_iphi=iphi1
c     Debug
            if (db.eq.1) then
               write(*,*) 'new_itheta=',new_itheta
               write(*,*) 'new_iphi=',new_iphi
            endif
c     Debug
            pos(1)=P1(2)
            pos(2)=P1(3)
            call extract_coeffs(new_itheta,new_iphi,
     &           coeff_a,coeff_b,coeff_c,
     &           ca,cb,cc)
            call ground_radius(pos,
     &           theta(new_itheta-1),theta(new_itheta),
     &           phi(new_iphi-1),phi(new_iphi),
     &           ca,cb,cc,r_ground)
c     Debug
            if (db.eq.1) then
               write(*,*) 'r_ground=',r_ground
            endif
c     Debug
            if (P1(1).le.r_ground) then
               ct_new=1
               surface_new=1
               new_ir=1
c     Debug
               if (db.eq.1) then
                  write(*,*) 'ct_new=',ct_new
                  write(*,*) 'surface_new=',surface_new
                  write(*,*) 'new_ir=',new_ir
               endif
c     Debug
               goto 123
            else if (P1(1).gt.
     &              r(Nr(new_itheta,new_iphi),new_itheta,new_iphi)) then
               ct_new=1
               surface_new=2
               new_ir=Nr(new_itheta,new_iphi)
c     Debug
               if (db.eq.1) then
                  write(*,*) 'ct_new=',ct_new
                  write(*,*) 'surface_new=',surface_new
                  write(*,*) 'new_ir=',new_ir
               endif
c     Debug
               goto 123
            else
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'Pstart=',(Pstart(i),i=1,dim)
               write(*,*) 'u_in=',(u_in(i),i=1,dim)
               write(*,*) 'ioe=',ioe
               write(*,*) 'P1=',(P1(i),i=1,dim)
               write(*,*) 'cellf=',cellf
               write(*,*) 'columnf=',columnf
               write(*,*) 'itheta1=',itheta1,' iphi1=',iphi1
               write(*,*) 'r_ground=',r_ground
               stop
            endif
         endif                  ! cellf
      
c     Debug
         if ((surface_new.lt.1).or.(surface_new.gt.6)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'surface_new=',surface_new
            write(*,*) 'position 2'
            stop
         endif
c     Debug

      endif                     ! ioe=1
      
      call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &     ca,cb,cc)

      if (ct.eq.1) then
         if (surface.eq.1) then
            ct_new=2
            surface_new=1
            new_ir=1
         else if (surface.eq.2) then
            ct_new=2
            surface_new=2
            new_ir=Nr(itheta,iphi)
         else
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'wrong input parameters:'
            write(*,*) 'ct=',ct
            write(*,*) 'surface=',surface
         stop
         endif
         if (ioe.eq.0) then
            new_itheta=itheta
            new_iphi=iphi
         else
         endif ! ios
      else if (ct.eq.2) then
         if (surface.eq.1) then
            if (ir.eq.1) then
               ct_new=1
               surface_new=1
            else
               ct_new=2
               surface_new=2
               call prev_r_cell(Nr,ir,itheta,iphi,new_ir)
            endif
            new_itheta=itheta
            new_iphi=iphi
         else if (surface.eq.2) then
            if (ir.eq.Nr(itheta,iphi)) then
               ct_new=1
               surface_new=2
            else
               ct_new=2
               surface_new=1
               call next_r_cell(Nr,ir,itheta,iphi,new_ir)
            endif
            new_itheta=itheta
            new_iphi=iphi
         else if (surface.eq.3) then
            ct_new=2
            surface_new=4
            new_itheta=itheta
            call prev_phi_cell(Nphi,iphi,new_iphi)
            call extract_coeffs(new_itheta,new_iphi,
     &           coeff_a,coeff_b,coeff_c,
     &           ca,cb,cc)
            call column_slide(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &           P,new_itheta,new_iphi,new_ir)
c            call column_slide(Nr,r,P(1),new_itheta,new_iphi,new_ir)
         else if (surface.eq.4) then
            ct_new=2
            surface_new=3
            new_itheta=itheta
            call next_phi_cell(Nphi,iphi,new_iphi)
            call extract_coeffs(new_itheta,new_iphi,
     &           coeff_a,coeff_b,coeff_c,
     &           ca,cb,cc)
            call column_slide(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &           P,new_itheta,new_iphi,new_ir)
c            call column_slide(Nr,r,P(1),new_itheta,new_iphi,new_ir)
         else if (surface.eq.5) then
            ct_new=2
            call prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           new_itheta,iphi_mod)
            if (itheta.eq.1) then
               new_iphi=iphi_mod
               surface_new=5
            else
               new_iphi=iphi
               surface_new=6
            endif
            call extract_coeffs(new_itheta,new_iphi,
     &           coeff_a,coeff_b,coeff_c,
     &           ca,cb,cc)
            call column_slide(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &           P,new_itheta,new_iphi,new_ir)
c            call column_slide(Nr,r,P(1),new_itheta,new_iphi,new_ir)
         else if (surface.eq.6) then
            ct_new=2
            call next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           new_itheta,iphi_mod)
            if (itheta.eq.Ntheta) then
               new_iphi=iphi_mod
               surface_new=6
            else
               new_iphi=iphi
               surface_new=5
            endif
            call extract_coeffs(new_itheta,new_iphi,
     &           coeff_a,coeff_b,coeff_c,
     &           ca,cb,cc)
            call column_slide(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &           P,new_itheta,new_iphi,new_ir)
c            call column_slide(Nr,r,P(1),new_itheta,new_iphi,new_ir)
         else
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'wrong input parameters:'
            write(*,*) 'ct=',ct
            write(*,*) 'surface=',surface
         stop
         endif
      else
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong input parameter ct=',ct
         stop
      endif

 123  continue
c     Debug
      if ((surface.lt.1).or.(surface.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface=',surface
         write(*,*) 'position 3'
         stop
      endif
      if ((surface_new.lt.1).or.(surface_new.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_new=',surface_new
         write(*,*) 'position 3'
         stop
      endif
c     Debug
      call realign_position(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     ct,surface,ir,itheta,iphi,ct_new,surface_new,
     &     new_ir,new_itheta,new_iphi,P)
      call copy_vector(dim,P,P_out)
c     Debug
      if (db.eq.1) then
         write(*,*) 'after realign_position:'
         write(*,*) 'new_ir=',new_ir
         write(*,*) 'new_itheta=',new_itheta
         write(*,*) 'new_iphi=',new_iphi
         write(*,*) 'P=',(P(i),i=1,dim)
         write(*,*) label(1:strlen(label)),' :out'
      endif
c     Debug
c     Debug
      if ((surface.lt.1).or.(surface.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface=',surface
         write(*,*) 'position 4'
         stop
      endif
      if ((surface_new.lt.1).or.(surface_new.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_new=',surface_new
         write(*,*) 'position 4'
         stop
      endif
c     Debug

      return
      end



      subroutine identify_cell(dim,P,Nr,Ntheta,Nphi,r,theta,phi,
     &     coeff_a,coeff_b,coeff_c,
     &     r_nodes,r_south_pole,r_north_pole,
     &     cf,ir,itheta,iphi)
      implicit none
      include 'max.inc'
c     
c     Purpose: to identify the cell the provided position is in
c
c     Inputs:
c       + dim: dimension of vectors
c       + P: (r,theta,phi) position
c       + Nr,Ntheta,Nphi: number of cells
c       + r,theta,phi: spherical discretization
c       + coeff_a,coeff_b,coeff_c: coefficients for computing ground radius
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c
c     Outputs:
c       + cf: cell was identified
c       + ir,itheta,iphi: index of the cell "P" was identified into
c

c     input
      integer dim
      double precision P(1:Nmat_mx)
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
c     output
      integer cf
      integer ir,itheta,iphi
c     temp
      double precision pos(1:2),r_ground
      integer i,app,inc
      integer thetai,phii,ri
      integer in
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)

      cf=0
      do thetai=1,Ntheta
         do phii=1,Nphi
            do ri=1,Nr(thetai,phii)
               call extract_coeffs(thetai,phii,coeff_a,coeff_b,coeff_c,
     &              ca,cb,cc)
               app=0
               if (ri.eq.1) then
                  call need2approximate(dim,Ntheta,r,theta,phi,
     &                 r_nodes,r_south_pole,r_north_pole,
     &                 thetai,phii,app)
               endif

               in=0
               if (app.eq.0) then
                  if ((P(1).ge.r(ri-1,thetai,phii)).and.
     &                 (P(1).le.r(ri,thetai,phii)).and.
     &                 (P(2).ge.theta(thetai-1)).and.
     &                 (P(2).le.theta(thetai)).and.
     &                 (P(3).ge.phi(phii-1)).and.
     &                 (P(3).le.phi(phii))) then
                     in=1
                  endif
               else
                  call is_in_column(P,theta(thetai-1),theta(thetai),
     &                 phi(phii-1),phi(phii),inc)
                  pos(1)=P(2)
                  pos(2)=P(3)
                  call ground_radius(pos,theta(thetai-1),theta(thetai),
     &                 phi(phii-1),phi(phii),ca,cb,cc,r_ground)
                  if ((inc.eq.1).and.(P(1).ge.r_ground-1.0D-4).and.
     &                 (P(1).le.r(ri,thetai,phii))) then
                     in=1
                  endif
               endif

               if (in.eq.1) then
                  cf=1
                  ir=ri
                  itheta=thetai
                  iphi=phii
                  goto 666
               endif

            enddo ! ri
         enddo ! phii
      enddo ! thetai

 666  continue
      return
      end



      subroutine identify_column(P,Ntheta,Nphi,theta,phi,
     &     cf,itheta,iphi)
      implicit none
      include 'max.inc'
c     
c     Purpose: to identify the column the provided position is in
c
c     Inputs:
c       + P: (r,theta,phi) position
c       + Ntheta,Nphi: number of cells
c       + theta,phi: spherical discretization
c
c     Outputs:
c       + cf: 1 when column was identified; 0 otherwise
c       + itheta,iphi: index of the cell "P" was identified into
c

c     input
      double precision P(1:Nmat_mx)
      integer Ntheta,Nphi
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
c     output
      integer cf
      integer itheta,iphi
c     temp
      integer thetai,phii
      integer in
      integer strlen
      character*100 label
      label='subroutine identify_column'

      cf=0
      do thetai=1,Ntheta
         do phii=1,Nphi
            if ((P(2).ge.theta(thetai-1)).and.(P(2).le.theta(thetai))
     &           .and.(P(3).ge.phi(phii-1)).and.(P(3).le.phi(phii)))
     &           then
               itheta=thetai
               iphi=phii
               cf=1
               goto 123
            endif
         enddo ! phii
      enddo ! thetai
 123  continue

      return
      end




      subroutine realign_position(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     ct,surface,ir,itheta,iphi,ct_new,surface_new,
     &     new_ir,new_itheta,new_iphi,P)
      implicit none
      include 'max.inc'
c
c     Purpose: to adjust precisely the position of the particle over an interface
c     from the previous and the next cells it was/is in
c     

c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer ct,surface
      integer ir,itheta,iphi
      integer ct_new,surface_new
      integer new_ir,new_itheta,new_iphi,iphi_mod
c     I/O
      double precision P(1:Nmat_mx)
c     temp
      integer app
      double precision pos(1:2)
      double precision r_ground
      integer strlen
      integer iphi_prev,iphi_next
      character*100 label
      label='subroutine realign_position'

      if (ct_new.eq.1) then ! if the particle arrives on a surface (ground or space)
         if (surface_new.eq.1) then ! this is ground
            call need2approximate(dim,Ntheta,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,
     &           new_itheta,new_iphi,app)
            if (app.eq.0) then
               P(1)=r(0,new_itheta,new_iphi)
            else
               pos(1)=P(2)
               pos(2)=P(3)
               call ground_radius(pos,
     &              theta(new_itheta-1),theta(new_itheta),
     &              phi(new_iphi-1),phi(new_iphi),
     &              ca,cb,cc,r_ground)
               P(1)=r_ground
c     Debug
c               write(*,*) 'From routine ',label(1:strlen(label)),' :'
c               write(*,*) 'r_ground1=',r_ground
c               write(*,*) new_itheta,new_iphi
c     Debug
            endif
         else ! this is space
            P(1)=r(Nr(new_itheta,new_iphi),new_itheta,new_iphi)
         endif
      else if ((ct_new.eq.2).and.(ct.eq.2)) then ! if the particle comes from a cell and arrives in a cell (volumic)
         if ((itheta.eq.new_itheta).and.(iphi.eq.new_iphi).
     &        and.(ir.ne.new_ir)) then
c     if the particle remains on the same column
            if (new_ir.eq.ir-1) then
               P(1)=r(new_ir,new_itheta,new_iphi)
            else if (new_ir.eq.ir+1) then
               P(1)=r(ir,new_itheta,new_iphi)
            else
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'error1'
               stop
            endif
         else if (iphi.eq.new_iphi) then
c     if the particle remains at the same longitude
            if (new_itheta.eq.itheta-1) then
               P(2)=theta(new_itheta)
            else if (new_itheta.eq.itheta+1) then
               P(2)=theta(itheta)
            endif
         else if (itheta.eq.new_itheta) then
c     if the particle remains at the same latitude
            call prev_phi_cell(Nphi,iphi,iphi_prev)
            call next_phi_cell(Nphi,iphi,iphi_next)
            if (new_iphi.eq.iphi_prev) then
               P(3)=phi(new_iphi)
            else if (new_iphi.eq.iphi_next) then
               P(3)=phi(new_iphi-1)
            else
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'error2'
               stop
            endif
         endif
      else if ((ct_new.eq.2).and.(ct.eq.1)) then ! if the particle comes from a surface and arrives in a cell (volumic)
         if (surface.eq.1) then ! the particle comes from the ground
            call need2approximate(dim,Ntheta,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,
     &           new_itheta,new_iphi,app)
            if (app.eq.0) then
               P(1)=r(0,new_itheta,new_iphi)
            else
               pos(1)=P(2)
               pos(2)=P(3)
               call ground_radius(pos,
     &              theta(new_itheta-1),theta(new_itheta),
     &              phi(new_iphi-1),phi(new_iphi),
     &              ca,cb,cc,r_ground)
               P(1)=r_ground
c     Debug
c               write(*,*) 'From routine ',label(1:strlen(label)),' :'
c               write(*,*) 'r_ground2=',r_ground
c               write(*,*) new_itheta,new_iphi
c     Debug
            endif
         else if (surface.eq.2) then ! the particle comes from space
            P(1)=r(Nr(new_itheta,new_iphi),new_itheta,new_iphi)
         endif
      endif

      return
      end




      subroutine prev_r_cell(Nr,ir,itheta,iphi,ir_prev)
      implicit none
      include 'max.inc'
c     purpose: to return 'ir_prev', the index of the previous
c     cell on a altitude row

c     inputs
      integer Nr(1:Ntheta_mx,1:Nphi_mx),ir,itheta,iphi
c     outputs
      integer ir_prev
c     temp
      integer strlen
      character*100 label
      label='subroutine prev_r_cell'

      if ((ir.le.0).or.(ir.ge.Nr(itheta,iphi)+1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for ir=',ir
         stop
      endif
      
      if (ir.le.1) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'No previous cell for ir=',ir
         stop
      endif

      ir_prev=ir-1

      return
      end



      subroutine next_r_cell(Nr,ir,itheta,iphi,ir_next)
      implicit none
      include 'max.inc'
c     purpose: to return 'ir_next', the index of the next
c     cell on a altitude row

c     inputs
      integer Nr(1:Ntheta_mx,1:Nphi_mx),ir,itheta,iphi
c     outputs
      integer ir_next
c     temp
      integer strlen
      character*100 label
      label='subroutine next_r_cell'

      if ((ir.le.0).or.(ir.ge.Nr(itheta,iphi)+1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for ir=',ir
         stop
      endif
      
      if (ir.ge.Nr(itheta,iphi)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'No next cell for ir=',ir
         stop
      endif

      ir_next=ir+1

      return
      end



      subroutine prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &     itheta_prev,iphi_mod)
      implicit none
c     purpose: to return 'itheta_prev', the index of the previous
c     cell on a latitude row

c     inputs
      integer Ntheta,Nphi,itheta,iphi
c     outputs
      integer itheta_prev,iphi_mod
c     temp
      integer strlen
      character*100 label
      label='subroutine prev_theta_cell'

      if ((itheta.le.0).or.(itheta.ge.Ntheta+1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for itheta=',itheta
         stop
      endif
      
      if (itheta.eq.1) then
         itheta_prev=1
         if (iphi.le.Nphi/2) then
            iphi_mod=iphi+Nphi/2
         else
            iphi_mod=iphi-Nphi/2
         endif
      else
         itheta_prev=itheta-1
      endif

      return
      end



      subroutine next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &     itheta_next,iphi_mod)
      implicit none
c     purpose: to return 'itheta_next', the index of the next
c     cell on a latitude row
      integer strlen
      integer Ntheta,Nphi,itheta,iphi,itheta_next,iphi_mod
      character*100 label
      label='subroutine next_theta_cell'

      if ((itheta.le.0).or.(itheta.ge.Ntheta+1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for itheta=',itheta
         stop
      endif
      
      if (itheta.eq.Ntheta) then
         itheta_next=Ntheta
         if (iphi.le.Nphi/2) then
            iphi_mod=iphi+Nphi/2
         else
            iphi_mod=iphi-Nphi/2
         endif
      else
         itheta_next=itheta+1
      endif

      return
      end



      subroutine prev_phi_cell(Nphi,iphi,iphi_prev)
      implicit none
c     purpose: to return 'iphi_prev', the index of the previous
c     cell on a longitude row
      integer strlen
      integer Nphi,iphi,iphi_prev
      character*100 label
      label='subroutine prev_phi_cell'

      if ((iphi.le.0).or.(iphi.ge.Nphi+1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for iphi=',iphi
         write(*,*) 'Nphi=',Nphi
         stop
      endif
      
      if (iphi.eq.1) then
         iphi_prev=Nphi
      else
         iphi_prev=iphi-1
      endif

      return
      end



      subroutine next_phi_cell(Nphi,iphi,iphi_next)
      implicit none
c     purpose: to return 'iphi_next', the index of the next
c     cell on a longitude row
      integer strlen
      integer Nphi,iphi,iphi_next
      character*100 label
      label='subroutine next_phi_cell'

      if ((iphi.le.0).or.(iphi.ge.Nphi+1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for iphi=',iphi
         stop
      endif
      
      if (iphi.eq.Nphi) then
         iphi_next=1
      else
         iphi_next=iphi+1
      endif

      return
      end



      subroutine column_slide(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     P,itheta,iphi,ir)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to compute the index 'ir' of the atmospheric layer the particle arrives in
c     when coming from a neighbour column
c
c     Inputs:
c       + dim: dimension of arrays
c       + Nr,Ntheta,Nphi
c       + r,theta,phi
c       + r_nodes: radiuses at grid nodes
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + ca,cb,cc: a, b and c coefficients for each quarter
c       + P: (r,theta,phi) position of the particle at column interface
c       + itheta,iphi: index of the column reached by the particle
c
c     Outputs:
c       + ir: index of the atmospheric layer (in the column) reached by the particle
c
c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision P(1:Nmat_mx)
      integer itheta,iphi
c     outputs
      integer ir
c     temp
      integer strlen
      double precision Pr
      integer i,irf,app
      double precision pos(1:2),r_ground,ground_level
      character*100 label
      label='subroutine column_slide'

      call need2approximate(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)

      if (app.eq.1) then
         pos(1)=P(2)
         pos(2)=P(3)
         call ground_radius(pos,theta(itheta-1),theta(itheta),
     &        phi(iphi-1),phi(iphi),
     &        ca,cb,cc,r_ground)
         ground_level=r_ground
      else
         ground_level=r(0,itheta,iphi)
      endif
      
      Pr=P(1)
      if (Pr.lt.ground_level) then
         if ((iphi.eq.1).and.(dabs(P(3)-2.0D+0*pi).lt.1.0D-8)) then
            pos(1)=P(2)
            pos(2)=0.0D+0
            call ground_radius(pos,theta(itheta-1),theta(itheta),
     &           phi(iphi-1),phi(iphi),
     &           ca,cb,cc,r_ground)
            if (Pr.lt.r_ground) then
c     Debug
               write(*,*) label(1:strlen(label)),
     &              ' : zground error iphi=',
     &              iphi
               write(*,*) 'P=',(P(i),i=1,dim)
               write(*,*) 'r_ground @ phi=0:',r_ground
               write(*,*) 'still not ok !'
            else
c     Debug
c               write(*,*) 'ok now'
c     Debug
            endif
         endif
         if ((iphi.eq.Nphi).and.(dabs(P(3)).lt.1.0D-8)) then
            pos(1)=P(2)
            pos(2)=2.0D+0*pi
            call ground_radius(pos,theta(itheta-1),theta(itheta),
     &           phi(iphi-1),phi(iphi),
     &           ca,cb,cc,r_ground)
            if (Pr.lt.r_ground) then
c     Debug
               write(*,*) label(1:strlen(label)),
     &              ' : zground error iphi=',
     &              iphi
               write(*,*) 'P=',(P(i),i=1,dim)
               write(*,*) 'r_ground @ phi=2pi:',r_ground
               write(*,*) 'still not ok !'
            else
c     Debug
c               write(*,*) 'ok now'
c     Debug
            endif
         endif
      endif
      if (Pr.gt.r(Nr(itheta,iphi),itheta,iphi)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Incorrect argument Pr=',Pr
         write(*,*) 'while r(',Nr(itheta,iphi),
     &        ',',itheta,',',iphi,')=',r(Nr(itheta,iphi),itheta,iphi)
         stop
      endif

      irf=0
      do i=1,Nr(itheta,iphi)
         if ((Pr.ge.r(i-1,itheta,iphi)).and.
     &        (Pr.le.r(i,itheta,iphi))) then
            ir=i
            irf=1
            goto 123
         endif
      enddo
 123  continue

      if ((irf.eq.0).and.(app.eq.1).and.
     &     (r_ground.lt.r(0,itheta,iphi)).and.
     &     (Pr.ge.r_ground-1.0D-4).and.(Pr.le.r(0,itheta,iphi))) then
         ir=1
         irf=1
      endif
      
      if (irf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'index of layer could not be found for:'
         write(*,*) 'itheta=',itheta
         write(*,*) 'iphi=',iphi
         write(*,*) 'Nr(',itheta,',',iphi,')=',Nr(itheta,iphi)
         write(*,*) 'Pr=',Pr
         write(*,*) 'app=',app
         if (app.eq.1) then
            write(*,*) 'r_ground=',r_ground
         endif
         do i=0,Nr(itheta,iphi)
            write(*,*) 'r(',i,')=',r(i,itheta,iphi)
         enddo
         write(*,*) 'P=',(P(i),i=1,dim)
         stop
      endif

      return
      end



      subroutine ir_tmp_cells(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     r_mc,T,P,itheta,iphi,
     &     rtmp_prev,rtmp,rtmp_next,Ttmp_prev,Ttmp,Ttmp_next)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute altitudes "rtmp_prev", "rtmp" and "rtmp_next"
c     at which temperatures must be taken for a given column (itheta,iphi)
c     at altitude P(1)
c

c     inputs
      integer dim
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision P(1:Nmat_mx)
      integer itheta,iphi
c     outputs
      double precision rtmp_prev,rtmp,rtmp_next
      double precision Ttmp_prev,Ttmp,Ttmp_next
c     temp
      integer strlen
      integer ir,ir_prev,ir_next
      character*100 label
      label='subroutine ir_tmp_cells'
      
      call column_slide(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
     &     P,itheta,iphi,ir)
c      call column_slide(Nr,r,Pr,itheta,iphi,ir)
      if (ir.gt.1) then
         call prev_r_cell(Nr,ir,itheta,iphi,ir_prev)
      endif
      if (ir.lt.Nr(itheta,iphi)) then
         call next_r_cell(Nr,ir,itheta,iphi,ir_next)
      endif

      rtmp=r_mc(ir,itheta,iphi)
      Ttmp=T(ir,itheta,iphi)
      if (ir.eq.1) then
         rtmp_prev=r(0,itheta,iphi)
         Ttmp_prev=T(0,itheta,iphi)
      else
         rtmp_prev=r_mc(ir_prev,itheta,iphi)
         Ttmp_prev=T(ir_prev,itheta,iphi)
      endif
      if (ir.eq.Nr(itheta,iphi)) then
         rtmp_next=r(Nr(itheta,iphi),itheta,iphi)
         Ttmp_next=T(Nr(itheta,iphi)+1,itheta,iphi)
      else
         rtmp_next=r_mc(ir_next,itheta,iphi)
         Ttmp_next=T(ir_next,itheta,iphi)
      endif
      
      return
      end



      subroutine s_angle(theta1,theta2,phi1,phi2,sa)
      implicit none
c
c     Purpose: to compute the solid angle within an angular sector
c     that is defined by its longitude and latitude limits
c
c     Inputs:
c       + theta1,theta2: latitude limits
c       + phi1,phi2: longitude limits
c
c     Outputs:
c       + sa: solid angle
c

c     input
      double precision theta1,theta2
      double precision phi1,phi2
c     output
      double precision sa
c     temp
      double precision surface_r
      double precision S,r
      integer strlen
      character*100 label
      label='subroutine sangle'

      r=1.0D+4 ! or any other value
      S=surface_r(r,theta1,theta2,phi1,phi2)
      sa=S/r**2.0D+0

      return
      end
