c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine surface_reflexion(dim,ref,Ntheta,Nphi,
     &     r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c,
     &     itheta,iphi,surface,P_in,u_in,
     &     ground_emissivity,space_emissivity,
     &     Nzones,emissivity_zone,emissivity,
     &     energy_in,weight_in,
     &     P_out,u_out,energy_out,weight_out)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to handle surface reflexions
c     
c     Inputs:
c       + dim: dimension of vectors
c       + ref: option for reflexion at ground level (1: specular, 2: diffuse)
c       + Ntheta,Nphi: number of cells in latitude & longitude
c       + r,theta,phi: cells discretization
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + coeff_a, coeff_b, coeff_c: a, b and c coefficients for each quarter
c       + itheta,iphi: index of the column where position "P"' is located
c       + surface: surface type (1:ground, 2:space)
c       + P_in: position on the surface
c       + u_in: propagation direction when the particle hits the surface
c       + ground_emissivity: emissivity of ground for column index (itheta,iphi)
c       + space_emissivity: emissivity of space for column index (itheta,iphi)
c       + Nzones: number of special emissivity zones
c       + emissivity_zone(izone,j): spatial dimensions of special emissivity zones
c         - (izone,1): minimum latitude (rad) of the zone
c         - (izone,2): maximum latitude (rad) of the zone
c         - (izone,3): minimum longitude (rad) of the zone
c         - (izone,4): maximum longitude (rad) of the zone
c       + emissivity: values of emissivity
c         - (izone): emissivity of the zone index 'izone'
c       + energy_in: energy carried by the particle when hitting the surface
c       + weight_in: statistical weight of the particle when hitting the surface
c     
c     Outputs:
c       + P_out: position of the particle when leaving the surface
c       + u_out: propagation direction when leaving the surface
c       + energy_out: energy carried by the particle after reflexion
c       + weight_out: statistical weight of the particle after reflexion
c

c     Inputs
      integer dim,ref
      integer Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
      integer itheta,iphi,surface
      double precision P_in(1:Nmat_mx)
      double precision u_in(1:Nmat_mx)
      integer Nzones
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      double precision emissivity(1:Nzones_mx)
      double precision energy_in,weight_in
c     Outputs
      double precision P_out(1:Nmat_mx)
      double precision u_out(1:Nmat_mx)
      double precision energy_out,weight_out
c     temp
      integer strlen,i,app
      double precision pdf0,pdf
      double precision n(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision local_emissivity
      double precision prod,raf
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      character*100 label
      label='subroutine surface_reflexion'

      call need2approximate(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
      
      if ((app.eq.1).and.(surface.eq.1)) then
         call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &        ca,cb,cc)
         call ground_normal(dim,P_in,theta(itheta-1),theta(itheta),
     &        phi(iphi-1),phi(iphi),ca,cb,cc,n)
      else
         call inside_normal(surface,P_in,n)
      endif
      call scalar_product(dim,u_in,n,prod)
c     the local normal and the incoming propagation direction should be in opposite directions
      if (prod.gt.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'propagation direction u_in=',(u_in(i),i=1,dim)
         write(*,*) 'and local normal n=',(n(i),i=1,dim)
         write(*,*) 'are not in agreement'
         write(*,*) 'scalar product:',prod
         write(*,*) 'P_in=',(P_in(i),i=1,dim)
         write(*,*) 'itheta=',itheta,' iphi=',iphi
         write(*,*) 'surface=',surface
         write(*,*) 'app=',app
         if ((app.eq.1).and.(surface.eq.1)) then
            write(*,*) 'normal computed using ground_normal'
            call inside_normal(surface,P_in,n)
            write(*,*) 'normal by inside_normal:',(n(i),i=1,dim)
         else
            write(*,*) 'normal computed using inside_normal'
         endif
         stop
      endif

c     compute ground emissivity at position "P_in"
      call find_emissivity(dim,Ntheta,Nphi,itheta,iphi,
     &     surface,P_in,ground_emissivity,space_emissivity,
     &     Nzones,emissivity_zone,emissivity,
     &     local_emissivity)
      
      if (ref.eq.1) then ! specular reflexion
c     rotation of vector "-u_in", by an angle pi, around axe "n" -> vector "u_out"
         call scalar_vector(dim,-1.0D+0,u_in,u)
         call rotation(dim,u,pi,n,u_out)
         pdf=1.0D+0
         pdf0=1.0D+0
      else if (ref.eq.2) then   ! diffuse reflexion
         call diffuse_emission(dim,n,u_out,pdf)
         call angular_physical_pdf(dim,u_out,n,pdf0)
      else
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'input argument ref=',ref
         write(*,*) 'has incorrect value'
         stop
      endif
      call copy_vector(dim,P_in,P_out)
      energy_out=energy_in*(1.0D+0-local_emissivity)
      weight_out=weight_in*pdf0/pdf

      return
      end




      subroutine find_emissivity(dim,Ntheta,Nphi,itheta,iphi,
     &     surface,P,ground_emissivity,space_emissivity,
     &     Nzones,emissivity_zone,emissivity,
     &     local_emissivity)
      include 'max.inc'
c
c     Purpose: to compute the emissivity at a given position
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: number of cells in latitude & longitude
c       + itheta,iphi: index of the column where position "P"' is located
c       + surface: surface type (1:ground, 2:space)
c       + P: position for which emissivity has to be retrieved
c       + ground_emissivity: emissivity of ground for column index (itheta,iphi)
c       + space_emissivity: emissivity of space for column index (itheta,iphi)
c       + Nzones: number of special emissivity zones
c       + emissivity_zone(izone,j): spatial dimensions of special emissivity zones
c         - (izone,1): minimum latitude (rad) of the zone
c         - (izone,2): maximum latitude (rad) of the zone
c         - (izone,3): minimum longitude (rad) of the zone
c         - (izone,4): maximum longitude (rad) of the zone
c       + emissivity: values of emissivity
c         - (izone): emissivity of the zone index 'izone'
c     
c     Outputs:
c       + local_emissivity: emissivity at position "P"
c

c     Inputs
      integer dim
      integer Ntheta,Nphi
      integer itheta,iphi,surface
      double precision P(1:Nmat_mx)
      integer Nzones
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      double precision emissivity(1:Nzones_mx)
c     Output
      double precision local_emissivity
c     temp
      integer strlen
      integer sz,izone,zone,j
      character*100 label
      label='subroutine find_emissivity'

      if (surface.eq.1) then
         sz=0
         do zone=1,Nzones
            if ((P(2).ge.emissivity_zone(zone,1)).and.
     &           (P(2).le.emissivity_zone(zone,2)).and.
     &           (P(3).ge.emissivity_zone(zone,3)).and.
     &           (P(3).le.emissivity_zone(zone,4))) then
               if (sz.eq.0) then
                  sz=1
                  izone=zone
                  local_emissivity=emissivity(izone)
               else
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'Position P:',(P(j),j=1,dim)
                  write(*,*) 'has been identified in both special'
                  write(*,*) 'emissivity zones:'
                  write(*,*) izone,(emissivity_zone(izone,j),j=1,4)
                  write(*,*) 'and:'
                  write(*,*) zone,(emissivity_zone(zone,j),j=1,4)
                  stop
               endif
            endif
         enddo                  ! zone
         
         if (sz.eq.0) then
            local_emissivity=ground_emissivity(itheta,iphi)
         endif
c     Debug
c         write(*,*) 'ground emissivity=',local_emissivity
c     Debug
      else if (surface.eq.2) then
         local_emissivity=space_emissivity(itheta,iphi)
c     Debug
c         write(*,*) 'space emissivity=',local_emissivity
c     Debug
      else
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'input parameter surface=',surface
         stop
      endif

      return
      end



      subroutine entry_position_from_space(dim,method,theta0,phi0,
     &     Nr,Ntheta,Nphi,r,theta,phi,rspace,
     &     itheta_in,iphi_in,P_in,u_in,w)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: to chose a solar entry position on the most external surface ("space" surface)
c
c     Inputs:
c       + dim: dimension of the vectors
c       + method: method used for entry position sampling (1,2)
c       + theta0: latitude of the point where the star is at the zenith (rad)
c       + phi0: longitude of the point where the star is at the zenith (rad)
c       + Nr,Ntheta,Nphi: number of cells in latitude/longitude
c       + r,theta,phi: limits of the spherical cells in latitude/longitude
c       + rspace: radius at space level
c
c     Outputs:
c       + itheta_in,iphi_in: index of the external type 2 surface the entry position belongs to
c       + P_in: (r,theta,phi) solar entry position
c       + u_in: solar entry direction (x,y,z)
c       + w: value of the weight associated with statistical sampling used (m)
c

c     inputs
      integer dim,method
      double precision theta0,phi0
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision rspace
c     outputs
      integer itheta_in,iphi_in
      double precision P_in(1:Nmat_mx)
      double precision u_in(1:Nmat_mx)
      double precision w
c     temp
      integer strlen,i
      double precision pdf
      double precision n_in(1:Nmat_mx)
      double precision P0(1:Nmat_mx)
      double precision u0(1:Nmat_mx)
      integer n_int
      double precision P_int0(1:Nmat_mx)
      double precision P_int1(1:Nmat_mx)
      double precision P_int2(1:Nmat_mx)
      double precision k0,k1,k2
      integer sf,ir,itheta,iphi
      double precision r1,r2,theta1,theta2,phi1,phi2
      integer ons,surface,inc
      double precision n(1:Nmat_mx),prod
      character*100 label
      label='subroutine entry_position_from_space'
c     method:
c     1: generate a position P0 on the planetary cross-section, then tilt it with the correct angle,
c     and then compute the intersection between the external sphere and the line that goes
c     from position P0 with direction u0 (direction of the sun)
c     2: generate uniformly a position on external sphere

c     computing solar direction "u0"
      call spher2cart(label,1,1.0D+0,theta0,phi0,u0(1),u0(2),u0(3))
      call scalar_vector(dim,-1.0D+0,u0,u_in)
c     choosing entry position "P_in" uniformly on the r(Nr) half-sphere
      if (method.eq.1) then
c     computing "P0", the position on the planet cross-section
         call cross_section(dim,rspace,theta0,phi0,P0,pdf)
c     WARNING: rspace is passed as input argument to routine "cross_section"
c     within the sole assumption that all altitudes of space are the same

c     "P_in" is the intersection between the external sphere of radius "R"
c     and the line defined by position "P0" and direction "u0"
         call line_sphere_intersect(0,dim,P0,u0,rspace,n_int,
     &        P_int0,k0,P_int1,k1,P_int2,k2)
         if (n_int.ne.2) then
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'number of intersections:',n_int
            stop
         endif
         if (k2.lt.k1) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),':'
            write(*,*) '2 solutions found with:'
            write(*,*) 'k1=',k1,' k2=',k2
            stop
         endif
         if (k1.gt.0.0D+0) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),':'
            write(*,*) 'first solution found with k1=',k1
            stop
         endif
         if (k2.lt.0.0D+0) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),':'
            write(*,*) 'second solution found with k2=',k2
            stop
         endif
         call copy_vector(dim,P_int2,P_in)
c         call inside_normal(2,P_in,n)
c         call scalar_product(dim,u_in,n,prod)
         w=1.0D+0/pdf ! m

      else if (method.eq.2) then
         call half_sphere_uniform(dim,rspace,
     &        theta0,phi0,P_in,pdf)
c     Debug
c         write(*,*) 'P_in:',(P_in(i),i=1,dim)
c     Debug
         w=1.0D+0/pdf

      else
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'method=',method
         stop
      endif

c     identification of the type 2 surface "P_in" belongs to
      sf=0
      do itheta=1,Ntheta
         do iphi=1,Nphi
            ir=Nr(itheta,iphi)
            r1=r(ir-1,itheta,iphi)
            r2=r(ir,itheta,iphi)
            theta1=theta(itheta-1)
            theta2=theta(itheta)
            phi1=phi(iphi-1)
            phi2=phi(iphi)
c     routine "is_on_surface" is too permissive: a given position can be detected on a surface
c     even if it is only close to it. So I have to add another level of "more strict" test.
            call is_in_column(P_in,theta1,theta2,phi1,phi2,inc)
c            if ((P_in(2).ge.theta1).and.(P_in(2).le.theta2).and.
c     &           (P_in(3).ge.phi1).and.(P_in(3).le.phi2)) then
            if (inc.eq.1) then
               call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &              P_in,ons,surface)
               if ((ons.eq.1).and.(surface.eq.2)) then
                  sf=1
                  itheta_in=itheta
                  iphi_in=iphi
                  goto 123
               endif
            endif
         enddo ! iphi
      enddo ! itheta
 123  continue
      if (sf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'External surface not found'
         stop
      endif

c     last check
      call inside_normal(2,P_in,n)
      call scalar_product(dim,u_in,n,prod)
      if (prod.lt.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'input position:',(P_in(i),i=1,dim)
         write(*,*) 'input direction:',(u_in(i),i=1,dim)
         write(*,*) 'normal at that point:',(n(i),i=1,dim)
         write(*,*) 'scalar product:',prod
         stop
      endif

      return
      end



      subroutine surfaces_on_ground(dim,Ntheta,Nphi,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c,ground_surfaces)
      implicit none
      include 'max.inc'
c
c     Purpose: to approximate the surface at ground level
c     for every column (angular sector)
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: latitude/longitude discretization
c       + r,theta,phi: limits of spherical cells
c       + r_nodes: radiuses at grid nodes
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + coeff_a,coeff_b,coeff_c: a, b and c coefficients for each quarter
c
c     Outputs:
c       + ground_surfaces: surface at ground level for every column
c

c     inputs
      integer dim,Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
c     outputs
      double precision ground_surfaces(1:Ntheta_mx,1:Nphi_mx)
c     temp
      integer itheta,iphi
      integer strlen
      integer app(1:Ntheta_mx,1:Nphi_mx)
      character*100 label
c     debug
      double precision spher_surf(1:Ntheta_mx,1:Nphi_mx)
      double precision surface_r
c     debug
      label='subroutine surfaces_on_ground'
 20   format(2I3,e17.9)
 21   format(2I3,2e17.9)

c      do itheta=1,Ntheta
      do itheta=1,Ntheta
         do iphi=1,Nphi
            spher_surf(itheta,iphi)=surface_r(r(0,itheta,iphi),
     &           theta(itheta-1),theta(itheta),
     &           phi(iphi-1),phi(iphi))
            call need2approximate(dim,Ntheta,r,theta,phi,
     &           r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &           app(itheta,iphi))
            if (app(itheta,iphi).eq.1) then
               call surface_4points(dim,theta,phi,itheta,iphi,
     &              coeff_a,coeff_b,coeff_c,
     &              ground_surfaces(itheta,iphi))
            else
               ground_surfaces(itheta,iphi)=spher_surf(itheta,iphi)
            endif
c     Debug
c            write(*,*) itheta,iphi,
c     &           spher_surf(itheta,iphi),
c     &           ground_surfaces(itheta,iphi)
c     Debug
         enddo                  ! iphi
      enddo                     ! itheta

c     debug
      open(10,file='./logs/ground_surfaces.log')
      write(10,*) 'itheta / iphi / spher. surface / real surface'
      do itheta=1,Ntheta
         do iphi=1,Nphi
            if (app(itheta,iphi).eq.0) then
               write(10,20) itheta,iphi,
     &              spher_surf(itheta,iphi)
            else
               write(10,21) itheta,iphi,
     &              spher_surf(itheta,iphi),
     &              ground_surfaces(itheta,iphi)
            endif
         enddo ! iphi
      enddo ! itheta
      close(10)
c     debug

      return
      end



      subroutine need2approximate(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
      implicit none
      include 'max.inc'
c     Purpose: to determiner whether it is necessary or not
c     to approximate the real ground surface within an angular sector
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta: latitude discretization
c       + r,theta,phi: limits of spherical cells
c       + r_nodes: radiuses at grid nodes
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + itheta,iphi: index of the angular sector
c     
c     Output:
c       + app:
c         - app=0: no need to approximate ground surface
c         - app=1: need to approximate ground surface
c

c     inputs
      integer dim,Ntheta
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      integer itheta,iphi
c     outputs
      integer app
c     temp
      integer i
      double precision rp(0:4),err_r_max
      integer strlen
      character*100 label
      label='subroutine need2approximate'

      call set_rp(Ntheta,r,r_nodes,r_south_pole,r_north_pole,
     &     itheta,iphi,rp)
      app=0

      err_r_max=1.0D-2
      do i=1,4
         if (dabs(rp(i)-rp(0)).gt.err_r_max) then
            app=1
c     Debug
c            write(*,*) 'From routine ',label(1:strlen(label)),' :'
c     Debug
            goto 123
         endif
      enddo
 123  continue

      return
      end


      
      subroutine set_rp(Ntheta,r,r_nodes,r_south_pole,r_north_pole,
     &     itheta,iphi,rp)
      implicit none
      include 'max.inc'
c
c     Purpose: to produce "rp", the array of known ground radiuses over the
c     current angular sector (over which the interpolation is performed)
c
c     Inputs:
c       + Ntheta: number of columns over the latitude
c       + r: limits of spherical cells in terms of radiuses
c       + r_nodes: radiuses at grid nodes
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + itheta,iphi: index of the angular sector
c     Outputs:
c       + rp: array of known radiuses
c         - rp(0): radius at the center of the angular sector
c         - rp(j),j=1,4: radiuses at each corner of the angular sector
c
c     inputs
      integer Ntheta
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      integer itheta,iphi
c     outputs
      double precision rp(0:4)
c     temp
      integer strlen
      character*100 label
      label='subroutine set_rp'

      rp(0)=r(0,itheta,iphi)
      if (itheta.eq.1) then
         rp(1)=r_nodes(itheta+1,iphi,2)
         rp(2)=r_south_pole
         rp(3)=r_south_pole
         rp(4)=r_nodes(itheta+1,iphi,3)
      else if (itheta.eq.Ntheta) then
         rp(1)=r_north_pole
         rp(2)=r_north_pole
         rp(3)=r_nodes(itheta-1,iphi,1)
         rp(4)=r_nodes(itheta-1,iphi,4)
      else
         rp(1)=r_nodes(itheta,iphi,1)
         rp(2)=r_nodes(itheta,iphi,2)
         rp(3)=r_nodes(itheta,iphi,3)
         rp(4)=r_nodes(itheta,iphi,4)
      endif
      
      return
      end




      subroutine surface_4points(dim,theta,phi,itheta,iphi,
     &     coeff_a,coeff_b,coeff_c,surface)
      implicit none
      include 'max.inc'
c
c     Purpose: to approximate the surface between 4 points on a given angular sector
c
c     Inputs:
c       + dim: dimension of vectors
c       + theta,phi: limits of spherical cells
c       + itheta,iphi: index of the angular sector
c       + coeff_a,coeff_b,coeff_c: a, b and c coefficients for each quarter
c
c     Output:
c       + surface: value of the ground surface for the sector
c

c     inputs
      integer dim
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      integer itheta,iphi
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
c     outputs
      double precision surface
c     temp
      integer i,j,Nint,l
      double precision P1(1:2)
      double precision P2(1:2)
      double precision P3(1:2)
      double precision P4(1:2)
      double precision r1,r2,r3,r4
      double precision l12,l23,l34,l41
      double precision S,dS,h,a
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer strlen
      character*100 label
      label='subroutine surface_4points'
      Nint=100
      call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &     ca,cb,cc)

      S=0.0D+0
      do i=1,Nint
         do j=1,Nint
c     point 1
            P1(1)=theta(itheta-1)+(theta(itheta)-theta(itheta-1))
     &           /dble(Nint)*dble(i-1)
            P1(2)=phi(iphi-1)+(phi(iphi)-phi(iphi-1))
     &           /dble(Nint)*dble(i-1)
            P2(1)=theta(itheta-1)+(theta(itheta)-theta(itheta-1))
     &           /dble(Nint)*dble(i)
            P2(2)=phi(iphi-1)+(phi(iphi)-phi(iphi-1))
     &           /dble(Nint)*dble(i-1)
            P3(1)=theta(itheta-1)+(theta(itheta)-theta(itheta-1))
     &           /dble(Nint)*dble(i)
            P3(2)=phi(iphi-1)+(phi(iphi)-phi(iphi-1))
     &           /dble(Nint)*dble(i)
            P4(1)=theta(itheta-1)+(theta(itheta)-theta(itheta-1))
     &           /dble(Nint)*dble(i-1)
            P4(2)=phi(iphi-1)+(phi(iphi)-phi(iphi-1))
     &           /dble(Nint)*dble(i)
            call ground_radius(P1,theta(itheta-1),theta(itheta),
     &           phi(itheta-1),phi(itheta),ca,cb,cc,r1)
            call ground_radius(P2,theta(itheta-1),theta(itheta),
     &           phi(itheta-1),phi(itheta),ca,cb,cc,r2)
            call ground_radius(P3,theta(itheta-1),theta(itheta),
     &           phi(itheta-1),phi(itheta),ca,cb,cc,r3)
            call ground_radius(P4,theta(itheta-1),theta(itheta),
     &           phi(itheta-1),phi(itheta),ca,cb,cc,r4)
            call distance_on_sphere(dim,P1,P2,(r1+r2)/2.0D+0,l12)
            call distance_on_sphere(dim,P2,P3,(r2+r3)/2.0D+0,l23)
            call distance_on_sphere(dim,P3,P4,(r3+r4)/2.0D+0,l34)
            call distance_on_sphere(dim,P4,P1,(r4+r1)/2.0D+0,l41)
            a=dabs(l41-l23)/2.0D+0
c     Debug
            if (l12**2.0D+0-a**2.0D+0.lt.0.0D+0) then
               write(*,*) 'Error from ',label(1:strlen(label)),
     &              ' :'
               write(*,*) 'L12^2-a^2=',l12**2.0D+0-a**2.0D+0
               write(*,*) 'L12=',l12
               write(*,*) 'a=',a
               stop
            endif
c     Debug
            h=dsqrt(l12**2.0D+0-a**2.0D+0)
            dS=(l41+l23)*h/2.0D+0
            S=S+dS
         enddo ! j
      enddo ! i
      surface=S
      
      return
      end



      subroutine reset_onground(P,theta1,theta2,phi1,phi2,ca,cb,cc)
      implicit none
      include 'max.inc'
c
c     Purpose: to reset the provided position on ground if needed
c
c     Inputs:
c       + P: (r,theta,phi) spherical coordinates of the position
c       + theta1,theta2,phi1,phi2: limits in theta and phi of the current column
c       + ca,cb,cc: a,b and c coefficients for each quarter of the current column
c     Outputs:
c       + updated position P, if needed
c

c     input
      double precision P(1:Nmat_mx)
      double precision theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     temp
      double precision pos(1:2)
      double precision r_ground
      double precision dz_max
      integer strlen
      character*100 label
      label='subroutine reset_onground'
      dz_max=0.0D+0 ! m

      pos(1)=P(2)
      pos(2)=P(3)
      call ground_radius(pos,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,r_ground)
      if (r_ground-P(1).gt.dz_max) then
         P(1)=r_ground
      endif

      return
      end





      subroutine ground_surface(dim,Ntheta,Nphi,r,theta,phi,r_nodes,
     &     r_south_pole,r_north_pole,itheta,iphi,S)
      implicit none
      include 'max.inc'
c     
c     Purpose: to approximate the surface of the ground over a (theta,phi)
c     angular sector
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: latitude/longitude discretization
c       + r,theta,phi: limits of spherical cells
c       + r_nodes: radiuses at grid nodes
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + itheta,iphi: index of the angular sector
c
c     Output:
c       + S: value of the ground surface for the sector
c

c     inputs
      integer dim,Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      integer itheta,iphi
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
c     outputs
      double precision S
c     temp
      double precision P(1:2)
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision P3(1:Nmat_mx)
      double precision rc,thetac,phic
      double precision surface1,surface2,surface3,surface4
      integer strlen,j
      character*100 label
      label='subroutine ground_surface'

c     latitude and longitude at the center of the angular sector
      thetac=(theta(itheta)+theta(itheta-1))/2.0D+0
      phic=(phi(iphi)+phi(iphi-1))/2.0D+0
      P(1)=thetac
      P(2)=phic
      call ground_radius(dim,Ntheta,r,theta,phi,r_nodes,
     &     r_south_pole,r_north_pole,itheta,iphi,P,rc)
      
      S=0.0D+0
      P1(1)=rc
      P1(2)=thetac
      P1(3)=phic
      if (itheta.eq.1) then
         P2(1)=r_nodes(itheta+1,iphi,2)
         P2(2)=theta(itheta)
         P2(3)=phi(iphi-1)
         P3(1)=r_nodes(itheta+1,iphi,3)
         P3(2)=theta(itheta)
         P3(3)=phi(iphi)
      else if (itheta.eq.Ntheta) then
         P2(1)=r_nodes(itheta-1,iphi,1)
         P2(2)=theta(itheta-1)
         P2(3)=phi(iphi)
         P3(1)=r_nodes(itheta-1,iphi,4)
         P3(2)=theta(itheta-1)
         P3(3)=phi(iphi-1)
      else
         P2(1)=r_nodes(itheta,iphi,1)
         P2(2)=theta(itheta)
         P2(3)=phi(iphi-1)
         P3(1)=r_nodes(itheta,iphi,4)
         P3(2)=theta(itheta)
         P3(3)=phi(iphi)
      endif
      call surface_3points(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &     P1,P2,P3,surface1)
c     Debug
c      write(*,*) 'surface1=',surface1
c     Debug
      if (itheta.eq.1) then
         P2(1)=r_south_pole
         P2(2)=theta(itheta-1)
         P2(3)=phi(iphi-1)
         P3(1)=r_nodes(itheta+1,iphi,2)
         P3(2)=theta(itheta)
         P3(3)=phi(iphi-1)
      else if (itheta.eq.Ntheta) then
         P2(1)=r_north_pole
         P2(2)=theta(itheta)
         P2(3)=phi(iphi-1)
         P3(1)=r_nodes(itheta-1,iphi,4)
         P3(2)=theta(itheta-1)
         P3(3)=phi(iphi-1)
      else
         P2(1)=r_nodes(itheta,iphi,2)
         P2(2)=theta(itheta-1)
         P2(3)=phi(iphi-1)
         P3(1)=r_nodes(itheta,iphi,1)
         P3(2)=theta(itheta)
         P3(3)=phi(iphi-1)
      endif
      call surface_3points(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &     P1,P2,P3,surface2)
c     Debug
c      write(*,*) 'surface2=',surface2
c     Debug
      if (itheta.eq.1) then
         surface3=0.0D+0
      else if (itheta.eq.Ntheta) then
         surface3=0.0D+0
      else
         P2(1)=r_nodes(itheta,iphi,3)
         P2(2)=theta(itheta-1)
         P2(3)=phi(iphi)
         P3(1)=r_nodes(itheta,iphi,2)
         P3(2)=theta(itheta-1)
         P3(3)=phi(iphi-1)
         call surface_3points(dim,Ntheta,r,theta,phi,
     &        r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &        P1,P2,P3,surface3)
      endif
c     Debug
c      write(*,*) 'surface3=',surface3
c     Debug
      if (itheta.eq.1) then
         P2(1)=r_south_pole
         P2(2)=theta(itheta-1)
         P2(3)=phi(iphi)
         P3(1)=r_nodes(itheta+1,iphi,3)
         P3(2)=theta(itheta)
         P3(3)=phi(iphi)
      else if (itheta.eq.Ntheta) then
         P2(1)=r_north_pole
         P2(2)=theta(itheta)
         P2(3)=phi(iphi)
         P3(1)=r_nodes(itheta-1,iphi,1)
         P3(2)=theta(itheta-1)
         P3(3)=phi(iphi)
      else
         P2(1)=r_nodes(itheta,iphi,4)
         P2(2)=theta(itheta)
         P2(3)=phi(iphi)
         P3(1)=r_nodes(itheta,iphi,3)
         P3(2)=theta(itheta-1)
         P3(3)=phi(iphi)
      endif
      call surface_3points(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &     P1,P2,P3,surface4)
c     Debug
c      write(*,*) 'surface4=',surface4
c     Debug
      S=surface1+surface2+surface3+surface4

      return
      end




      subroutine surface_3points(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &     P1,P2,P3,surface)
      implicit none
      include 'max.inc'
c
c     Purpose: to approximate the surface between 3 points on a given angular sector
c     NOTE: point 1 must be at the center of the angular sector, or at a pole
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: latitude/longitude discretization
c       + r,theta,phi: limits of spherical cells
c       + r_nodes: radiuses at grid nodes
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + itheta,iphi: index of the angular sector
c       + P1: (r,theta,phi) of the first point
c       + P2: (r,theta,phi) of the second point
c       + P3: (r,theta,phi) of the third point
c
c     Output:
c       + surface: value of the ground surface for the sector
c

c     inputs
      integer dim,Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole
      double precision r_north_pole
      integer itheta,iphi
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision P3(1:Nmat_mx)
c     outputs
      double precision surface
c     temp
      integer Nint,i,j
      double precision point1(1:2)
      double precision point2(1:2)
      double precision point3(1:2)
      double precision point4(1:2)
      double precision r1,r2,r3,r4
      double precision l12,l23,l34,l41
      double precision h,a,dS
      integer strlen
      character*100 label
      label='subroutine 3points_surface'

c     number of intervals for computation
      Nint=100
      surface=0.0D+0
      do i=1,Nint
c     point 1
         point1(1)=P2(2)-(P2(2)-P1(2))/dble(Nint)*dble(i-1)
         point1(2)=P2(3)+(P1(3)-P2(3))/dble(Nint)*dble(i-1)
         call ground_radius(dim,Ntheta,r,theta,phi,r_nodes,
     &        r_south_pole,r_north_pole,itheta,iphi,point1,r1)
c     point 2
         point2(1)=P2(2)-(P2(2)-P1(2))/dble(Nint)*dble(i)
         point2(2)=P2(3)+(P1(3)-P2(3))/dble(Nint)*dble(i)
         call ground_radius(dim,Ntheta,r,theta,phi,r_nodes,
     &        r_south_pole,r_north_pole,itheta,iphi,point2,r2)
c     point 3
         point3(1)=P3(2)-(P3(2)-P1(2))/dble(Nint)*dble(i)
         point3(2)=P3(3)-(P3(3)-P1(3))/dble(Nint)*dble(i)
         call ground_radius(dim,Ntheta,r,theta,phi,r_nodes,
     &        r_south_pole,r_north_pole,itheta,iphi,point3,r3)
c     point 4
         point4(1)=P3(2)-(P3(2)-P1(2))/dble(Nint)*dble(i-1)
         point4(2)=P3(3)-(P3(3)-P1(3))/dble(Nint)*dble(i-1)
         call ground_radius(dim,Ntheta,r,theta,phi,r_nodes,
     &        r_south_pole,r_north_pole,itheta,iphi,point4,r4)
c     distances between points
         call distance_on_sphere(dim,point1,point2,(r1+r2)/2.0D+0,l12)
         call distance_on_sphere(dim,point2,point3,(r2+r3)/2.0D+0,l23)
         call distance_on_sphere(dim,point3,point4,(r3+r4)/2.0D+0,l34)
         call distance_on_sphere(dim,point4,point1,(r4+r1)/2.0D+0,l41)
c     surface element
         a=(l41-l23)/2.0D+0
c     Debug
         if (a.lt.0.0D+0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'a=',a
            write(*,*) 'L41=',l41
            write(*,*) 'L23=',l23
            write(*,*) 'i=',i,' /',nint
            write(*,*) 'point1=',(point1(j),j=1,2)
            write(*,*) 'point2=',(point2(j),j=1,2)
            write(*,*) 'point3=',(point3(j),j=1,2)
            write(*,*) 'point4=',(point4(j),j=1,2)
            write(*,*) 'P1=',(P1(j),j=1,3)
            write(*,*) 'P2=',(P2(j),j=1,3)
            write(*,*) 'P3=',(P3(j),j=1,3)
            stop
         endif
         if (l12**2.0D+0-a**2.0D+0.lt.0.0D+0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'L12^2-a^2=',l12**2.0D+0-a**2.0D+0
            write(*,*) 'L12=',l12
            write(*,*) 'a=',a
            stop
         endif
c     Debug
         h=dsqrt(l12**2.0D+0-a**2.0D+0)
         dS=h*L41-a*h
c     increment total surface
         surface=surface+dS
      enddo ! i
      
      return
      end
      
      
      
      
      subroutine find_surface_index(r1,r2,theta1,theta2,phi1,phi2,
     &     coord,index)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify the surface index to which the specified point belongs to
c     
c     Inputs:
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c       + coord: (r,theta,phi) spherical coordinates of the point
c
c     output:
c       + index: index of the surface of the grid point "coord" belongs to
c
      
c     input
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision coord(1:Nmat_mx)
c     output
      integer index
c     temp
      character*100 label
      integer strlen
      double precision dr,dtheta,dphi
      integer sf
      parameter(dr=1.0D-3)
      parameter(dtheta=1.0D-6)
      parameter(dphi=1.0D-6)

      label='subroutine find_surface_index'
      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)

      sf=0
      if (dabs(coord(1)-r1).lt.dr) then
         index=1
         sf=1
      endif
      if (dabs(coord(1)-r2).lt.dr) then
         index=2
         sf=1
      endif
      if (dabs(coord(2)-phi1).lt.dphi) then
         index=3
         sf=1
      endif
      if (dabs(coord(2)-phi2).lt.dphi) then
         index=4
         sf=1
      endif
      if (dabs(coord(2)-theta1).lt.dtheta) then
         index=5
         sf=1
      endif
      if (dabs(coord(2)-theta2).lt.dtheta) then
         index=6
         sf=1
      endif

      if (sf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'surface index could not be identified'
         write(*,*) 'r1=',r1,' r2=',r2
         write(*,*) 'theta1=',theta1,' theta2=',theta2
         write(*,*) 'phi1=',phi1,' phi2=',phi2
         write(*,*) 'r=',coord(1),' theta=',coord(2),' phi=',coord(3)
         stop
      endif

      return
      end


      double precision function surface_box(r1,r2,theta1,theta2,
     &     phi1,phi2)
      implicit none
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision surface_r
      double precision surface_theta
      double precision surface_phi
      character*100 label
c
c     Purpose: to compute the total external surface of a given sphere portion
c     defined by radiuses r1 and r2, latitude angles theta1 and theta2,
c     longitude angles phi1 and phi2
c
c     Inputs:
c       + r1: first radius (m)
c       + r2: second radius (m), must be greater or equal to r1
c       + theta1: first latitude angle (rad)
c       + theta2: second latitude angle (rad), must be greater or equal to theta1
c       + phi1: first longitude angle (rad)
c       + phi2: second longitude angle (rad), must be greater or equal to phi1
c
c     Output:
c       + surface_box: surface (m)
c     

      label='subroutine surface_box'
      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)

      surface_box=surface_r(r1,theta1,theta2,phi1,phi2)
     &     +surface_r(r2,theta1,theta2,phi1,phi2)
     &     +2.0D+0*surface_theta(r1,r2,theta1,theta2)
     &     +surface_phi(theta1,r1,r2,phi1,phi2)
     &     +surface_phi(theta2,r1,r2,phi1,phi2)

      return
      end



      double precision function surface_r(r,theta1,theta2,phi1,phi2)
      implicit none
      double precision r,theta1,theta2,phi1,phi2
      character*100 label
c
c     Purpose: to compute the surface on a sphere of radius r, defined
c     by latitude angles theta1 and theta2, and longitude angles phi1 and phi2
c
c     Inputs:
c       + r: radius of the sphere (m)
c       + theta1: first latitude angle (rad)
c       + theta2: second latitude angle (rad), must be greater or equal to theta1
c       + phi1: first longitude angle (rad)
c       + phi2: second longitude angle (rad), must be greater or equal to phi1
c     
c     Output:
c       + surface_r: surface (m)
c
      label='subroutine surface_r'
      call test_surface_r(r,theta1,theta2,phi1,phi2,label)

      surface_r=r**2.0D+0*(phi2-phi1)*(dsin(theta2)-dsin(theta1))

      return
      end



      double precision function surface_theta(r1,r2,theta1,theta2)
      implicit none
      double precision r1,r2,theta1,theta2
      character*100 label
c
c     Purpose: to compute the lateral surface defined by radiuses r1 and r2
c     and latitude angles theta1 and theta2
c
c     Inputs:
c       + r1: first radius (m)
c       + r2: second radius (m), must be greater or equal to r1
c       + theta1: first latitude angle (rad)
c       + theta2: second latitude angle (rad), must be greater or equal to theta1
c
c     Output:
c       + surface_theta: surface (m)
c
      label='subroutine surface_theta'
      call test_surface_theta(r1,r2,theta1,theta2,label)
      surface_theta=(r2**2.0D+0-r1**2.0D+0)*(theta2-theta1)/2.0D+0

      return
      end




      double precision function surface_phi(theta,r1,r2,phi1,phi2)
      implicit none
      double precision theta,r1,r2,phi1,phi2
      character*100 label
c
c     Purpose: to compute the lateral surface defined by radiuses r1 and r2
c     and longitude angles phi1 and phi2, at latitude theta
c
c     Inputs:
c       + theta: latitude (rad)
c       + r1: first radius (m)
c       + r2: second radius (m), must be greater or equal to r1
c       + phi1: first longitude angle (rad)
c       + phi2: second longitude angle (rad), must be greater or equal to phi1
c
c     Output:
c       + surface_phi: surface (m)
c
      label='subroutine surface_phi'
      call test_surface_phi(theta,r1,r2,phi1,phi2,label)
      surface_phi=(r2**2.0D+0-r1**2.0D+0)*(phi2-phi1)
     &     *dcos(theta)/2.0D+0

      return
      end




      subroutine test_box(r1,r2,theta1,theta2,phi1,phi2,label)
      implicit none
      double precision r1,r2,theta1,theta2,phi1,phi2
      character*100 label

      call test_r(r1,label)
      call test_r(r2,label)
      call test_theta(theta1,label)
      call test_phi(phi1,label)
      call test_theta(theta2,label)
      call test_phi(phi2,label)

      call test_r_diff(r1,r2,label)
      call test_theta_diff(theta1,theta2,label)
      call test_phi_diff(phi1,phi2,label)

      return
      end




      subroutine test_surface_phi(theta,r1,r2,phi1,phi2,label)
      implicit none
      double precision r1,r2,theta,phi1,phi2
      character*100 label

      call test_theta(theta,label)
      call test_r(r1,label)
      call test_r(r2,label)
      call test_phi(phi1,label)
      call test_phi(phi2,label)

      call test_r_diff(r1,r2,label)
      call test_phi_diff(phi1,phi2,label)

      return
      end



      subroutine test_surface_theta(r1,r2,theta1,theta2,label)
      implicit none
      double precision r1,r2,theta1,theta2
      character*100 label
      
      call test_r(r1,label)
      call test_r(r2,label)
      call test_theta(theta1,label)
      call test_theta(theta2,label)

      call test_r_diff(r1,r2,label)
      call test_theta_diff(theta1,theta2,label)

      return
      end



      subroutine test_surface_r(r,theta1,theta2,phi1,phi2,label)
      implicit none
      double precision r,theta1,theta2,phi1,phi2
      character*100 label

      call test_r(r,label)
      call test_theta(theta1,label)
      call test_phi(phi1,label)
      call test_theta(theta2,label)
      call test_phi(phi2,label)
      
      call test_theta_diff(theta1,theta2,label)
      call test_phi_diff(phi1,phi2,label)
      
      return
      end



      subroutine test_r(r,label)
      implicit none
      double precision r
      character*100 label
      integer strlen

      if (r.lt.0.0D+0) then
         write(*,*) 'Error from function ',label(1:strlen(label)),':'
         write(*,*) 'r=',r
         stop
      endif

      return
      end



      subroutine test_theta(theta,label)
      implicit none
      include 'param.inc'
      double precision theta
      character*100 label
      integer strlen

      if ((theta.gt.pi/2.0D+0).or.(theta.lt.-pi/2.0D+0)) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'theta=',theta
         stop
      endif

      return
      end



      subroutine test_phi(phi,label)
      implicit none
      include 'param.inc'
      double precision phi
      double precision epsilon
      character*100 label
      integer strlen
      epsilon=1.0D-6

      if ((phi.lt.-epsilon).or.
     &     (phi-2.0D+0*pi.gt.epsilon)) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'phi=',phi
         stop
      endif

      return
      end



      subroutine test_r_diff(r1,r2,label)
      implicit none
      double precision r1,r2
      character*100 label
      integer strlen

      if (r2.lt.r1) then
         write(*,*) 'Error from function ',label(1:strlen(label)),':'
         write(*,*) 'r2=',r2,' < r1=',r1
         stop
      endif

      return
      end



      subroutine test_theta_diff(theta1,theta2,label)
      implicit none
      double precision theta1,theta2
      character*100 label
      integer strlen

      if (theta2.lt.theta1) then
         write(*,*) 'Error from function ',label(1:strlen(label)),':'
         write(*,*) 'theta2=',theta2,' < theta1=',theta1
         stop
      endif

      return
      end



      subroutine test_phi_diff(phi1,phi2,label)
      implicit none
      double precision phi1,phi2
      character*100 label
      integer strlen

      if (phi2.lt.phi1) then
         write(*,*) 'Error from function ',label(1:strlen(label)),':'
         write(*,*) 'phi2=',phi2,' < phi1=',phi1
         stop
      endif

      return
      end
